import functools
from spaceone.api.monitoring.v1 import data_source_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.monitoring.model.data_source_model import DataSource

__all__ = ['DataSourceInfo', 'DataSourcesInfo']


def PluginInfo(vo):
    if vo:
        info = {
            'plugin_id': vo.plugin_id,
            'version': vo.version,
            'options': change_struct_type(vo.options),
            'metadata': change_struct_type(vo.metadata),
            'secret_id': vo.secret_id,
            'provider': vo.provider,
            'upgrade_mode': vo.upgrade_mode
        }

        return data_source_pb2.DataSourcePluginInfo(**info)
    else:
        return None


def DataSourceInfo(data_source_vo: DataSource, minimal=False):
    info = {
        'data_source_id': data_source_vo.data_source_id,
        'name': data_source_vo.name,
        'state': data_source_vo.state,
        'monitoring_type': data_source_vo.monitoring_type,
        'provider': data_source_vo.provider
    }

    if not minimal:
        info.update({
            'capability': change_struct_type(data_source_vo.capability),
            'plugin_info': PluginInfo(data_source_vo.plugin_info),
            'tags': change_struct_type(data_source_vo.tags),
            'domain_id': data_source_vo.domain_id,
            'created_at': utils.datetime_to_iso8601(data_source_vo.created_at)
        })

    return data_source_pb2.DataSourceInfo(**info)


def DataSourcesInfo(data_source_vos, total_count, **kwargs):
    return data_source_pb2.DataSourcesInfo(results=list(
        map(functools.partial(DataSourceInfo, **kwargs), data_source_vos)), total_count=total_count)



import logging

from spaceone.core.manager import BaseManager
from spaceone.monitoring.model.webhook_model import Webhook
from spaceone.monitoring.manager.plugin_manager import PluginManager
from spaceone.monitoring.connector.webhook_plugin_connector import WebhookPluginConnector

_LOGGER = logging.getLogger(__name__)


class WebhookPluginManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.wp_connector: WebhookPluginConnector = self.locator.get_connector('WebhookPluginConnector')

    def initialize(self, endpoint):
        _LOGGER.debug(f'[initialize] webhook plugin endpoint: {endpoint}')
        self.wp_connector.initialize(endpoint)

    def init_plugin(self, options):
        plugin_info = self.wp_connector.init(options)

        _LOGGER.debug(f'[plugin_info] {plugin_info}')
        plugin_metadata = plugin_info.get('metadata', {})

        return plugin_metadata

    def verify_plugin(self, options):
        self.wp_connector.verify(options)

    def parse_event(self, options, data):
        return self.wp_connector.parse_event(options, data)

    def get_webhook_plugin_endpoint_by_vo(self, webhook_vo: Webhook):
        plugin_info = webhook_vo.plugin_info.to_dict()
        endpoint, updated_version = self.get_webhook_plugin_endpoint(plugin_info, webhook_vo.domain_id)

        if updated_version:
            _LOGGER.debug(f'[get_webhook_plugin_endpoint_by_vo] upgrade plugin version: {plugin_info["version"]} -> {updated_version}')
            self.upgrade_webhook_plugin_version(webhook_vo, endpoint, updated_version)

        return endpoint

    def get_webhook_plugin_endpoint(self, plugin_info, domain_id):
        plugin_mgr: PluginManager = self.locator.get_manager('PluginManager')
        return plugin_mgr.get_plugin_endpoint(plugin_info, domain_id)

    def upgrade_webhook_plugin_version(self, webhook_vo: Webhook, endpoint, updated_version):
        plugin_info = webhook_vo.plugin_info.to_dict()
        self.initialize(endpoint)
        plugin_metadata = self.init_plugin(plugin_info.get('options', {}))
        plugin_info['version'] = updated_version
        plugin_info['metadata'] = plugin_metadata
        webhook_vo.update({'plugin_info': plugin_info})



import functools
from spaceone.api.monitoring.v1 import maintenance_window_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.monitoring.model.maintenance_window_model import MaintenanceWindow

__all__ = ['MaintenanceWindowInfo', 'MaintenanceWindowsInfo']


def MaintenanceWindowInfo(maintenance_window_vo: MaintenanceWindow, minimal=False):
    info = {
        'maintenance_window_id': maintenance_window_vo.maintenance_window_id,
        'title': maintenance_window_vo.title,
        'state': maintenance_window_vo.state,
        'projects': maintenance_window_vo.projects,
        'start_time': utils.datetime_to_iso8601(maintenance_window_vo.start_time),
        'end_time': utils.datetime_to_iso8601(maintenance_window_vo.end_time)
    }

    if not minimal:
        info.update({
            'tags': change_struct_type(maintenance_window_vo.tags),
            'domain_id': maintenance_window_vo.domain_id,
            'created_by': maintenance_window_vo.created_by,
            'created_at': utils.datetime_to_iso8601(maintenance_window_vo.created_at),
            'updated_at': utils.datetime_to_iso8601(maintenance_window_vo.updated_at),
            'closed_at': utils.datetime_to_iso8601(maintenance_window_vo.closed_at)
        })

    return maintenance_window_pb2.MaintenanceWindowInfo(**info)


def MaintenanceWindowsInfo(maintenance_window_vos, total_count, **kwargs):
    return maintenance_window_pb2.MaintenanceWindowsInfo(results=list(
        map(functools.partial(MaintenanceWindowInfo, **kwargs), maintenance_window_vos)), total_count=total_count)



import functools
from typing import List
from spaceone.api.monitoring.v1 import alert_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils

from spaceone.monitoring.model.alert_model import Alert, Responder

__all__ = ['AlertInfo', 'AlertsInfo']


def RespondersInfo(vos: List[Responder]):
    if vos:
        responders = []
        for vo in vos:
            info = {
                'resource_type': vo.resource_type,
                'resource_id': vo.resource_id
            }
            responder = alert_pb2.AlertResponder(**info)
            responders.append(responder)

        return responders
    else:
        return None


def AlertResourceInfo(vo):
    if vo:
        info = {
            'resource_id': vo.resource_id,
            'resource_type': vo.resource_type,
            'name': vo.name
        }

        return alert_pb2.AlertResource(**info)
    else:
        return None


def AlertInfo(alert_vo: Alert, minimal=False):
    info = {
        'alert_number': alert_vo.alert_number,
        'alert_id': alert_vo.alert_id,
        'title': alert_vo.title,
        'state': alert_vo.state,
        'status_message': alert_vo.status_message,
        'assignee': alert_vo.assignee,
        'urgency': alert_vo.urgency,
        'escalation_step': alert_vo.escalation_step,
        'escalation_ttl': alert_vo.escalation_ttl,
        'project_id': alert_vo.project_id
    }

    if not minimal:
        info.update({
            'description': alert_vo.description,
            'severity': alert_vo.severity,
            'rule': alert_vo.rule,
            'image_url': alert_vo.image_url,
            'resource': AlertResourceInfo(alert_vo.resource),
            'provider': alert_vo.provider,
            'account': alert_vo.account,
            'additional_info': change_struct_type(alert_vo.additional_info),
            'is_snoozed': alert_vo.is_snoozed,
            'snoozed_end_time': utils.datetime_to_iso8601(alert_vo.snoozed_end_time),
            'responders': RespondersInfo(alert_vo.responders),
            'project_dependencies': alert_vo.project_dependencies,
            'triggered_by': alert_vo.triggered_by,
            'webhook_id': alert_vo.webhook_id,
            'escalation_policy_id': alert_vo.escalation_policy_id,
            'domain_id': alert_vo.domain_id,
            'created_at': utils.datetime_to_iso8601(alert_vo.created_at),
            'updated_at': utils.datetime_to_iso8601(alert_vo.updated_at),
            'acknowledged_at': utils.datetime_to_iso8601(alert_vo.acknowledged_at),
            'resolved_at': utils.datetime_to_iso8601(alert_vo.resolved_at),
            'escalated_at': utils.datetime_to_iso8601(alert_vo.escalated_at)
        })

    return alert_pb2.AlertInfo(**info)


def AlertsInfo(alert_vos, total_count, **kwargs):
    return alert_pb2.AlertsInfo(results=list(
        map(functools.partial(AlertInfo, **kwargs), alert_vos)), total_count=total_count)



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel


class MaintenanceWindow(MongoModel):
    maintenance_window_id = StringField(max_length=40, generate_id='mw', unique=True)
    title = StringField()
    state = StringField(max_length=20, default='OPEN', choices=('OPEN', 'CLOSED'))
    projects = ListField(StringField(max_length=40))
    start_time = DateTimeField()
    end_time = DateTimeField()
    tags = DictField()
    domain_id = StringField(max_length=40)
    created_by = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)
    updated_at = DateTimeField(auto_now=True)
    closed_at = DateTimeField(default=None, null=True)

    meta = {
        'updatable_fields': [
            'title',
            'state',
            'projects',
            'start_time',
            'end_time',
            'tags',
            'closed_at'
        ],
        'minimal_fields': [
            'maintenance_window_id',
            'title',
            'state',
            'projects',
            'start_time',
            'end_time'
        ],
        'change_query_keys': {
            'project_id': 'projects',
            'user_projects': 'projects'
        },
        'ordering': [
            '-start_time'
        ],
        'indexes': [
            # 'maintenance_window_id',
            'state',
            'projects',
            'start_time',
            'end_time',
            'created_by',
            'domain_id'
        ]
    }



import logging

from spaceone.core.manager import BaseManager
from spaceone.monitoring.model.alert_model import Alert, AlertNumber
from spaceone.monitoring.error.alert import *

_LOGGER = logging.getLogger(__name__)


class AlertManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.alert_model: Alert = self.locator.get_model('Alert')
        self.alert_number_model: AlertNumber = self.locator.get_model('AlertNumber')

    def create_alert(self, params):
        def _rollback(alert_vo):
            _LOGGER.info(f'[create_alert._rollback] '
                         f'Delete alert : #{str(alert_vo.alert_number)} '
                         f'({alert_vo.alert_id})')
            alert_vo.delete()

        params['alert_number'] = self._get_alert_number(params['domain_id'])
        params['alert_number_str'] = str(params['alert_number'])
        alert_vo: Alert = self.alert_model.create(params)
        self.transaction.add_rollback(_rollback, alert_vo)

        return alert_vo

    def update_alert(self, params):
        alert_vo: Alert = self.get_alert(params['alert_id'], params['domain_id'])
        return self.update_alert_by_vo(params, alert_vo)

    def update_alert_by_vo(self, params, alert_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[update_alert_by_vo._rollback] Revert Data : '
                         f'{old_data["alert_id"]}')
            alert_vo.update(old_data)

        self.transaction.add_rollback(_rollback, alert_vo.to_dict())
        return alert_vo.update(params)

    def add_responder(self, params):
        resource_type = params['resource_type']
        resource_id = params['resource_id']

        alert_vo: Alert = self.get_alert(params['alert_id'], params['domain_id'])

        for responder in alert_vo.responders:
            if responder.resource_type == resource_type and responder.resource_id == resource_id:
                raise ERROR_SAME_RESPONDER_ALREADY_EXISTS(resource_type=resource_type, resource_id=resource_id)

        alert_vo = alert_vo.append('responders', {'resource_type': resource_type, 'resource_id': resource_id})
        return alert_vo

    def remove_responder(self, params):
        resource_type = params['resource_type']
        resource_id = params['resource_id']

        alert_vo: Alert = self.get_alert(params['alert_id'], params['domain_id'])

        for responder in alert_vo.responders:
            if responder.resource_type == resource_type and responder.resource_id == resource_id:
                alert_vo = alert_vo.remove('responders', {'resource_type': resource_type, 'resource_id': resource_id})
                return alert_vo

        raise ERROR_RESPONDER_NOT_EXIST(resource_type=resource_type, resource_id=resource_id)

    def add_project_dependency(self, params):
        project_id = params['project_id']

        alert_vo: Alert = self.get_alert(params['alert_id'], params['domain_id'])

        for project_dependency in alert_vo.project_dependencies:
            if project_dependency == project_id:
                raise ERROR_SAME_PROJECT_DEPENDENCY_ALREADY_EXISTS(project_id=project_id)

        alert_vo = alert_vo.append('project_dependencies', project_id)
        return alert_vo

    def remove_project_dependency(self, params):
        project_id = params['project_id']

        alert_vo: Alert = self.get_alert(params['alert_id'], params['domain_id'])

        for project_dependency in alert_vo.project_dependencies:
            if project_dependency == project_id:
                alert_vo = alert_vo.remove('project_dependencies', project_id)
                return alert_vo

        raise ERROR_PROJECT_DEPENDENCY_NOT_EXIST(project_id=project_id)

    def delete_alert(self, alert_id, domain_id):
        alert_vo: Alert = self.get_alert(alert_id, domain_id)
        alert_vo.delete()

    def get_alert(self, alert_id, domain_id, only=None):
        return self.alert_model.get(alert_id=alert_id, domain_id=domain_id, only=only)

    def list_alerts(self, query={}):
        return self.alert_model.query(**query)

    def stat_alerts(self, query):
        return self.alert_model.stat(**query)

    def _get_alert_number(self, domain_id):
        def _rollback(alert_number_vo: AlertNumber):
            _LOGGER.info(f'[_get_alert_number._rollback] Decrement Number: {alert_number_vo.next}')
            alert_number_vo.decrement('next', 1)

        alert_number_vos = self.alert_number_model.filter(domain_id=domain_id)
        if alert_number_vos.count() > 0:
            account_number_vo = alert_number_vos[0].increment('next', 1)
            self.transaction.add_rollback(_rollback, account_number_vo)
        else:
            account_number_vo = self.alert_number_model.create({'domain_id': domain_id})
            self.transaction.add_rollback(_rollback, account_number_vo)

        return account_number_vo.next



import copy
import logging

from spaceone.core import cache
from spaceone.core.service import *
from spaceone.monitoring.error.escalation_policy import *
from spaceone.monitoring.model.escalation_policy_model import EscalationPolicy
from spaceone.monitoring.manager.identity_manager import IdentityManager
from spaceone.monitoring.manager.escalation_policy_manager import EscalationPolicyManager
from spaceone.monitoring.conf.default_escalation_policy import DEFAULT_ESCALATION_POLICY

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class EscalationPolicyService(BaseService):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.escalation_policy_mgr: EscalationPolicyManager = self.locator.get_manager('EscalationPolicyManager')

    @transaction(append_meta={
        'authorization.scope': 'DOMAIN_OR_PROJECT',
        'authorization.require_project_id': True
    })
    @check_required(['name', 'rules', 'domain_id'])
    def create(self, params):
        """Create escalation policy

        Args:
            params (dict): {
                'name': 'str',
                'rules': 'list',
                'repeat_count': 'int',
                'finish_condition': 'str',
                'project_id': 'str',
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            escalation_policy_vo (object)
        """

        project_id = params.get('project_id')
        domain_id = params['domain_id']

        identity_mgr: IdentityManager = self.locator.get_manager('IdentityManager')

        if project_id:
            identity_mgr.get_project(project_id, domain_id)
            params['scope'] = 'PROJECT'
        else:
            params['scope'] = 'DOMAIN'

        return self.escalation_policy_mgr.create_escalation_policy(params)

    @transaction(append_meta={'authorization.scope': 'DOMAIN_OR_PROJECT'})
    @check_required(['escalation_policy_id', 'domain_id'])
    def update(self, params):
        """Update escalation policy

        Args:
            params (dict): {
                'escalation_policy_id': 'dict',
                'name': 'str',
                'rules': 'list',
                'repeat_count': 'int',
                'finish_condition': 'str',
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            escalation_policy_vo (object)
        """

        escalation_policy_id = params['escalation_policy_id']
        domain_id = params['domain_id']

        if 'rules' in params:
            params['repeat_count'] = params.get('repeat_count', 0)

        escalation_policy_vo = self.escalation_policy_mgr.get_escalation_policy(escalation_policy_id, domain_id)
        return self.escalation_policy_mgr.update_escalation_policy_by_vo(params, escalation_policy_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN_OR_PROJECT'})
    @check_required(['escalation_policy_id', 'domain_id'])
    def set_default(self, params):
        """ Get escalation policy

        Args:
            params (dict): {
                'escalation_policy_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            escalation_policy_vo (object)
        """

        escalation_policy_id = params['escalation_policy_id']
        domain_id = params['domain_id']

        escalation_policy_vo: EscalationPolicy = self.escalation_policy_mgr.get_escalation_policy(escalation_policy_id,
                                                                                                  domain_id)

        if escalation_policy_vo.scope != 'DOMAIN':
            raise ERROR_INVALID_ESCALATION_POLICY_SCOPE(escalation_policy_id=escalation_policy_id)

        if escalation_policy_vo.is_default:
            return escalation_policy_vo
        else:
            return self.escalation_policy_mgr.set_default_escalation_policy(params, escalation_policy_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN_OR_PROJECT'})
    @check_required(['escalation_policy_id', 'domain_id'])
    def delete(self, params):
        """Delete escalation policy

        Args:
            params (dict): {
                'escalation_policy_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """

        self.escalation_policy_mgr.delete_escalation_policy(params['escalation_policy_id'], params['domain_id'])

    @transaction(append_meta={'authorization.scope': 'DOMAIN_OR_PROJECT'})
    @check_required(['escalation_policy_id', 'domain_id'])
    def get(self, params):
        """ Get escalation policy

        Args:
            params (dict): {
                'escalation_policy_id': 'str',
                'domain_id': 'str',
                'only': 'list
            }

        Returns:
            escalation_policy_vo (object)
        """

        return self.escalation_policy_mgr.get_escalation_policy(params['escalation_policy_id'],
                                                                params['domain_id'], params.get('only'))

    @transaction(append_meta={'authorization.scope': 'DOMAIN_OR_PROJECT'})
    @check_required(['domain_id'])
    @append_query_filter(['escalation_policy_id', 'name', 'is_default', 'finish_condition', 'scope',
                          'project_id', 'domain_id', 'user_projects'])
    @append_keyword_filter(['escalation_policy_id', 'name'])
    def list(self, params):
        """ List escalation polices

        Args:
            params (dict): {
                'escalation_policy_id': 'str',
                'name': 'str',
                'is_default': 'bool',
                'finish_condition': 'str',
                'scope': 'str',
                'project_id': 'str',
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)',
                'user_projects': 'list', // from meta
            }

        Returns:
            escalation_policy_vos (object)
            total_count
        """

        self._create_default_escalation_policy(params['domain_id'])
        query = params.get('query', {})
        return self.escalation_policy_mgr.list_escalation_policies(query)

    @transaction(append_meta={'authorization.scope': 'DOMAIN_OR_PROJECT'})
    @check_required(['query', 'domain_id'])
    @append_query_filter(['domain_id', 'user_projects'])
    @append_keyword_filter(['escalation_policy_id', 'name'])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',
                'user_projects': 'list', // from meta
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get('query', {})
        return self.escalation_policy_mgr.stat_escalation_policies(query)

    @cache.cacheable(key='escalation-policy:{domain_id}:default:init', expire=300)
    def _create_default_escalation_policy(self, domain_id):
        query = {
            'count_only': True,
            'filter': [
                {
                    'k': 'domain_id',
                    'v': domain_id,
                    'o': 'eq'
                }
            ]
        }
        escalation_policy_vos, total_count = self.escalation_policy_mgr.list_escalation_policies(query)
        if total_count == 0:
            default_escalation_policy = copy.deepcopy(DEFAULT_ESCALATION_POLICY)
            default_escalation_policy['domain_id'] = domain_id

            self.escalation_policy_mgr.create_escalation_policy(default_escalation_policy)

        return True



import logging
from datetime import datetime

from spaceone.core.service import *
from spaceone.core import cache, config
from spaceone.monitoring.error.alert import *
from spaceone.monitoring.model.alert_model import Alert
from spaceone.monitoring.model.project_alert_config_model import ProjectAlertConfig
from spaceone.monitoring.model.escalation_policy_model import EscalationPolicy
from spaceone.monitoring.manager.project_alert_config_manager import ProjectAlertConfigManager
from spaceone.monitoring.manager.escalation_policy_manager import EscalationPolicyManager
from spaceone.monitoring.manager.alert_manager import AlertManager
from spaceone.monitoring.manager.event_manager import EventManager
from spaceone.monitoring.manager.job_manager import JobManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler(exclude=['update_state'])
@authorization_handler(exclude=['update_state'])
@mutation_handler
@event_handler
class AlertService(BaseService):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.alert_mgr: AlertManager = self.locator.get_manager('AlertManager')
        self.event_mgr: EventManager = self.locator.get_manager('EventManager')

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['title', 'project_id', 'domain_id'])
    def create(self, params):
        """Create alert

        Args:
            params (dict): {
                'title': 'str',
                'description': 'str',
                'assignee': 'str',
                'urgency': 'str',
                'project_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            alert_vo (object)
        """

        project_id = params['project_id']
        domain_id = params['domain_id']

        project_alert_config_mgr: ProjectAlertConfigManager = self.locator.get_manager('ProjectAlertConfigManager')

        project_alert_config_vo: ProjectAlertConfig = project_alert_config_mgr.get_project_alert_config(project_id,
                                                                                                        domain_id)
        escalation_policy_vo: EscalationPolicy = project_alert_config_vo.escalation_policy

        params['escalation_policy_id'] = escalation_policy_vo.escalation_policy_id
        params['escalation_ttl'] = escalation_policy_vo.repeat_count + 1
        params['escalated_at'] = None

        # TODO: Check Assignee

        params['triggered_by'] = self.transaction.get_meta('user_id')

        alert_vo = self.alert_mgr.create_alert(params)

        self._create_notification(alert_vo, 'create_alert_notification')

        return alert_vo

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['alert_id', 'domain_id'])
    def update(self, params):
        """Update alert

        Args:
            params (dict): {
                'alert_id': 'str',
                'title': 'str',
                'state': 'str',
                'status_message': 'str',
                'description': 'str',
                'assignee': 'str',
                'urgency': 'str',
                'project_id': 'str',
                'reset_status_message': 'bool',
                'reset_description': 'bool',
                'reset_assignee': 'bool',
                'domain_id': 'str'
            }

        Returns:
            alert_vo (object)
        """

        alert_id = params['alert_id']
        domain_id = params['domain_id']
        project_id = params.get('project_id')
        state = params.get('state')
        assignee = params.get('assignee')
        status_message = params.get('status_message')
        reset_status_message = params.get('reset_status_message', False)
        reset_description = params.get('reset_description', False)
        reset_assignee = params.get('reset_assignee', False)

        is_resolved_notify = False
        is_assignee_notify = False

        if project_id:
            project_alert_config_mgr: ProjectAlertConfigManager = self.locator.get_manager('ProjectAlertConfigManager')

            project_alert_config_vo: ProjectAlertConfig = project_alert_config_mgr.get_project_alert_config(project_id,
                                                                                                            domain_id)
            escalation_policy_vo: EscalationPolicy = project_alert_config_vo.escalation_policy

            params['escalation_policy_id'] = escalation_policy_vo.escalation_policy_id
            params['escalation_ttl'] = escalation_policy_vo.repeat_count
            params['escalation_step'] = 1
            params['escalated_at'] = None
            params['assignee'] = None
            assignee = None

        if state:
            if state == 'ACKNOWLEDGED':
                params['acknowledged_at'] = datetime.utcnow()
                params['resolved_at'] = None
            elif state == 'RESOLVED':
                params['escalation_ttl'] = 0
                params['resolved_at'] = datetime.utcnow()
            elif state == 'TRIGGERED':
                params['acknowledged_at'] = None
                params['resolved_at'] = None

        alert_vo = self.alert_mgr.get_alert(alert_id, domain_id)

        if alert_vo.state == 'ERROR':
            raise ERROR_INVALID_PARAMETER(key='state', reason='The error state cannot be changed.')

        if alert_vo.state != state and state == 'RESOLVED' and status_message is None:
            params['status_message'] = ''

        if reset_status_message:
            params['status_message'] = ''

        if reset_description:
            params['description'] = ''

        if reset_assignee:
            params['assignee'] = None
            assignee = None

        if assignee:
            # TODO: Check Assignee
            is_assignee_notify = True

        if alert_vo.state in ['TRIGGERED', 'ACKNOWLEDGED'] and state == 'RESOLVED':
            is_resolved_notify = True

        updated_alert_vo: Alert = self.alert_mgr.update_alert_by_vo(params, alert_vo)

        if is_resolved_notify:
            self._create_notification(updated_alert_vo, 'create_resolved_notification')
        elif is_assignee_notify and updated_alert_vo.state in ['TRIGGERED', 'ACKNOWLEDGED']:
            self._create_notification(updated_alert_vo, 'create_assigned_notification', assignee)

        return updated_alert_vo

    @transaction
    @check_required(['alert_id', 'access_key', 'state'])
    def update_state(self, params):
        """Update alert state

        Args:
            params (dict): {
                'alert_id': 'str',
                'access_key': 'str',
                'state': 'str'
            }

        Returns:
            alert_vo (object)
        """
        alert_id = params['alert_id']
        access_key = params['access_key']
        state = params['state']

        is_resolved_notify = False

        # Check Access Key
        domain_id = self._check_access_key(alert_id, access_key)

        # Check State
        self._check_state(state)

        alert_vo = self.alert_mgr.get_alert(alert_id, domain_id)

        if alert_vo.state == 'ERROR':
            raise ERROR_INVALID_PARAMETER(key='state', reason='The error state cannot be changed.')

        if alert_vo.state != 'TRIGGERED':
            # raise ERROR_ALERT_ALREADY_PROCESSED(alert_id=alert_id)
            return alert_vo

        update_params = {
            'state': state
        }

        if state == 'ACKNOWLEDGED':
            update_params['acknowledged_at'] = datetime.utcnow()
        elif state == 'RESOLVED':
            update_params['resolved_at'] = datetime.utcnow()

        if alert_vo.state in ['TRIGGERED', 'ACKNOWLEDGED'] and state == 'RESOLVED':
            is_resolved_notify = True

        updated_alert_vo = self.alert_mgr.update_alert_by_vo(update_params, alert_vo)

        if is_resolved_notify:
            self._create_notification(updated_alert_vo, 'create_resolved_notification')

        return updated_alert_vo

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['alerts', 'merge_to', 'domain_id'])
    def merge(self, params):
        """Merge alerts

        Args:
            params (dict): {
                'alerts': 'list',
                'merge_to': 'str',
                'domain_id': 'str',
            }

        Returns:
            alert_vo (object)
        """
        merge_to = params['merge_to']
        alerts = params['alerts']
        domain_id = params['domain_id']

        self._check_merge_condition(merge_to=merge_to, alert_ids=alerts)
        alerts.remove(merge_to)

        events = []

        for alert_id in alerts:
            event_vos = self.event_mgr.filter_events(alert_id=alert_id, domain_id=domain_id)
            events += event_vos

        alert_vo = self.alert_mgr.get_alert(merge_to, domain_id)
        update_event_params = {'alert_id': merge_to, 'alert': alert_vo}

        for event_vo in events:
            self.event_mgr.update_event_by_vo(params=update_event_params, event_vo=event_vo)

        for alert_id in alerts:
            self.alert_mgr.delete_alert(alert_id=alert_id, domain_id=params['domain_id'])

        return alert_vo

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['alert_id', 'end_time', 'domain_id'])
    @change_timestamp_value(['end_time'], timestamp_format='iso8601')
    def snooze(self, params):
        """Snooze alert

        Args:
            params (dict): {
                'alert_id': 'str',
                'end_time': 'str',
                'domain_id': 'str'
            }

        Returns:
            alert_vo (object)
        """

        alert_id = params['alert_id']
        domain_id = params['domain_id']

        alert_vo = self.alert_mgr.get_alert(alert_id, domain_id)

        # TODO: Check end_times

        params['is_snoozed'] = True
        params['snoozed_end_time'] = params['end_time']

        return self.alert_mgr.update_alert_by_vo(params, alert_vo)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['alert_id', 'resource_type', 'resource_id', 'domain_id'])
    def add_responder(self, params):
        """Add alert responder

        Args:
            params (dict): {
                'alert_id': 'str',
                'resource_type': 'str',
                'resource_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            alert_vo (object)
        """

        # TODO: Check resource_type and resource_id

        return self.alert_mgr.add_responder(params)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['alert_id', 'resource_type', 'resource_id', 'domain_id'])
    def remove_responder(self, params):
        """Remove alert responder

        Args:
            params (dict): {
                'alert_id': 'str',
                'resource_type': 'str',
                'resource_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            alert_vo (object)
        """

        return self.alert_mgr.remove_responder(params)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['alert_id', 'project_id', 'domain_id'])
    def add_project_dependency(self, params):
        """Add dependent project

        Args:
            params (dict): {
                'alert_id': 'str',
                'project_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            alert_vo (object)
        """

        # TODO: Check project_id

        return self.alert_mgr.add_project_dependency(params)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['alert_id', 'project_id', 'domain_id'])
    def remove_project_dependency(self, params):
        """Remove dependent project

        Args:
            params (dict): {
                'alert_id': 'str',
                'project_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            alert_vo (object)
        """

        return self.alert_mgr.remove_project_dependency(params)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['alert_id', 'domain_id'])
    def delete(self, params):
        """Delete alert

        Args:
            params (dict): {
                'alert_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """

        self.alert_mgr.delete_alert(params['alert_id'], params['domain_id'])

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['alert_id', 'domain_id'])
    def get(self, params):
        """ Get alert

        Args:
            params (dict): {
                'alert_id': 'str',
                'domain_id': 'str',
                'only': 'list
            }

        Returns:
            alert_vo (object)
        """

        return self.alert_mgr.get_alert(params['alert_id'], params['domain_id'], params.get('only'))

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['domain_id'])
    @append_query_filter(['alert_number', 'alert_id', 'title', 'state', 'assignee', 'urgency', 'severity', 'is_snoozed',
                          'resource_id', 'triggered_by', 'webhook_id', 'escalation_policy_id', 'project_id',
                          'domain_id', 'user_projects'])
    @append_keyword_filter(['alert_id', 'alert_number_str', 'title'])
    def list(self, params):
        """ List alerts

        Args:
            params (dict): {
                'alert_number': 'str',
                'alert_id': 'str',
                'title': 'str',
                'state': 'str',
                'assignee': 'str',
                'urgency': 'str',
                'severity': 'str',
                'is_snoozed': 'bool',
                'resource_id': 'str',
                'webhook_id': 'bool',
                'escalation_policy_id': 'str',
                'project_id': 'str',
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)',
                'user_projects': 'list', // from meta
            }

        Returns:
            alert_vos (object)
            total_count
        """

        query = params.get('query', {})
        return self.alert_mgr.list_alerts(query)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['query', 'domain_id'])
    @append_query_filter(['domain_id', 'user_projects'])
    @append_keyword_filter(['alert_id', 'title'])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',
                'user_projects': 'list', // from meta
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get('query', {})
        return self.alert_mgr.stat_alerts(query)

    def _create_notification(self, alert_vo: Alert, method, user_id=None):
        params = {
            'alert_id': alert_vo.alert_id,
            'domain_id': alert_vo.domain_id
        }

        if user_id:
            params['user_id'] = user_id

        job_mgr: JobManager = self.locator.get_manager('JobManager')
        job_mgr.push_task(
            'monitoring_alert_notification_from_manual',
            'JobService',
            method,
            params
        )

    @staticmethod
    def _check_access_key(alert_id, access_key):
        domain_id = cache.get(f'alert-notification-callback:{alert_id}:{access_key}')

        if domain_id is None:
            raise ERROR_PERMISSION_DENIED()

        return domain_id

    @staticmethod
    def _check_state(state):
        if state not in ['ACKNOWLEDGED', 'RESOLVED']:
            raise ERROR_INVALID_PARAMETER(key='state', reason='Unsupported state. (ACKNOWLEDGED | RESOLVED)')

    @staticmethod
    def _check_merge_condition(merge_to, alert_ids):
        if merge_to not in alert_ids:
            raise ERROR_MERGE_ALERT_NOT_EXIST(alert_id=merge_to)



from spaceone.api.monitoring.v1 import log_pb2
from spaceone.core.pygrpc.message_type import *

__all__ = ['LogDataInfo']


def LogDataInfo(log_data):
    info = {
        'results': change_list_value_type(log_data['results']),
        'domain_id': log_data['domain_id']
    }

    return log_pb2.LogDataInfo(**info)



DEFAULT_ESCALATION_POLICY = {
    'name': 'Default',
    'is_default': True,
    'rules': [
        {
            'notification_level': 'ALL',
            'escalate_minutes': 0
        }
    ],
    'repeat_count': 0,
    'finish_condition': 'ACKNOWLEDGED',
    'scope': 'DOMAIN'
}



ROUTER = [
    {
        'router_path': 'spaceone.monitoring.interface.rest.v1.alert:router',
        'router_options': {
            'prefix': '/monitoring/v1',
        }
    },
    {
        'router_path': 'spaceone.monitoring.interface.rest.v1.event:router',
        'router_options': {
            'prefix': '/monitoring/v1',
        }
    },
]


from spaceone.api.monitoring.v1 import maintenance_window_pb2, maintenance_window_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class MaintenanceWindow(BaseAPI, maintenance_window_pb2_grpc.MaintenanceWindowServicer):

    pb2 = maintenance_window_pb2
    pb2_grpc = maintenance_window_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('MaintenanceWindowService', metadata) as maintenance_window_service:
            return self.locator.get_info('MaintenanceWindowInfo', maintenance_window_service.create(params))

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('MaintenanceWindowService', metadata) as maintenance_window_service:
            return self.locator.get_info('MaintenanceWindowInfo', maintenance_window_service.update(params))

    def close(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('MaintenanceWindowService', metadata) as maintenance_window_service:
            return self.locator.get_info('MaintenanceWindowInfo', maintenance_window_service.close(params))

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('MaintenanceWindowService', metadata) as maintenance_window_service:
            return self.locator.get_info('MaintenanceWindowInfo', maintenance_window_service.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('MaintenanceWindowService', metadata) as maintenance_window_service:
            maintenance_window_vos, total_count = maintenance_window_service.list(params)
            return self.locator.get_info('MaintenanceWindowsInfo',
                                         maintenance_window_vos,
                                         total_count,
                                         minimal=self.get_minimal(params))

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('MaintenanceWindowService', metadata) as maintenance_window_service:
            return self.locator.get_info('StatisticsInfo', maintenance_window_service.stat(params))



from spaceone.api.monitoring.v1 import project_alert_config_pb2, project_alert_config_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class ProjectAlertConfig(BaseAPI, project_alert_config_pb2_grpc.ProjectAlertConfigServicer):

    pb2 = project_alert_config_pb2
    pb2_grpc = project_alert_config_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectAlertConfigService', metadata) as project_alert_config_service:
            return self.locator.get_info('ProjectAlertConfigInfo', project_alert_config_service.create(params))

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectAlertConfigService', metadata) as project_alert_config_service:
            return self.locator.get_info('ProjectAlertConfigInfo', project_alert_config_service.update(params))

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectAlertConfigService', metadata) as project_alert_config_service:
            project_alert_config_service.delete(params)
            return self.locator.get_info('EmptyInfo')

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectAlertConfigService', metadata) as project_alert_config_service:
            return self.locator.get_info('ProjectAlertConfigInfo', project_alert_config_service.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectAlertConfigService', metadata) as project_alert_config_service:
            project_alert_config_vos, total_count = project_alert_config_service.list(params)
            return self.locator.get_info('ProjectAlertConfigsInfo',
                                         project_alert_config_vos,
                                         total_count,
                                         minimal=self.get_minimal(params))

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectAlertConfigService', metadata) as project_alert_config_service:
            return self.locator.get_info('StatisticsInfo', project_alert_config_service.stat(params))



import logging
from typing import List
from datetime import datetime, timedelta

from spaceone.core.error import *
from spaceone.core import queue, utils, config
from spaceone.core.manager import BaseManager
from spaceone.monitoring.model.job_model import Job

_LOGGER = logging.getLogger(__name__)


class JobManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.job_model: Job = self.locator.get_model('Job')
        self.job_timeout = config.get_global('JOB_TIMEOUT', 600)

    def is_domain_job_running(self, domain_id):
        job_vos: List[Job] = self.job_model.filter(domain_id=domain_id, status='IN_PROGRESS')

        running_job_count = job_vos.count()

        for job_vo in job_vos:
            if job_vo.remained_tasks == 0 and job_vo.status not in ['ERROR', 'TIMEOUT']:
                self.change_success_status(job_vo)
                running_job_count -= 1

            if datetime.utcnow() > (job_vo.created_at + timedelta(seconds=self.job_timeout)):
                self.change_timeout_status(job_vo)
                running_job_count -= 1

        if running_job_count > 0:
            return True
        else:
            return False

    def create_job(self, domain_id):
        return self.job_model.create({'domain_id': domain_id})

    def get_job(self, job_id, domain_id):
        return self.job_model.get(job_id=job_id, domain_id=domain_id)

    @staticmethod
    def decrease_remained_tasks(job_vo: Job):
        job_vo.decrement('remained_tasks', 1)

    @staticmethod
    def change_success_status(job_vo: Job):
        job_vo.delete()
        # job_vo.update({
        #     'status': 'SUCCESS',
        #     'finished_at': datetime.utcnow()
        # })

    @staticmethod
    def change_timeout_status(job_vo: Job):
        _LOGGER.error(f'Job Timeout ({job_vo.job_id}): {job_vo.domain_id}')

        job_vo.update({
            'status': 'TIMEOUT',
            'finished_at': datetime.utcnow()
        })

    @staticmethod
    def change_error_status(job_vo: Job, e):
        if not isinstance(e, ERROR_BASE):
            e = ERROR_UNKNOWN(message=str(e))

        _LOGGER.error(f'Job Error ({job_vo.job_id}): {e.message}', exc_info=True)

        job_vo.update({
            'status': 'ERROR',
            'finished_at': datetime.utcnow()
        })

        job_vo.append('errors', {
            'error_code': e.error_code,
            'message': e.message
        })

    def push_task(self, task_name, class_name, method, params):
        task = {
            'name': task_name,
            'version': 'v1',
            'executionEngine': 'BaseWorker',
            'stages': [{
                'locator': 'SERVICE',
                'name': class_name,
                'metadata': self.transaction.meta,
                'method': method,
                'params': {
                    'params': params
                }
            }]
        }

        queue.put('monitoring_q', utils.dump_json(task))



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel


class EventRuleCondition(EmbeddedDocument):
    key = StringField(required=True)
    value = StringField(required=True)
    operator = StringField(choices=('eq', 'contain', 'not', 'not_contain'))


class EventRuleOptions(EmbeddedDocument):
    stop_processing = BooleanField(default=False)


class EventRule(MongoModel):
    event_rule_id = StringField(max_length=40, generate_id='er', unique=True)
    name = StringField(max_length=255, default='')
    order = IntField(required=True)
    conditions = ListField(EmbeddedDocumentField(EventRuleCondition))
    conditions_policy = StringField(max_length=20, choices=('ALL', 'ANY', 'ALWAYS'))
    actions = DictField()
    options = EmbeddedDocumentField(EventRuleOptions, default=EventRuleOptions)
    tags = DictField()
    scope = StringField(max_length=20, choices=('GLOBAL', 'PROJECT'))
    project_id = StringField(max_length=40, default=None, null=True)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        'updatable_fields': [
            'name',
            'order',
            'conditions',
            'conditions_policy',
            'actions',
            'options',
            'tags'
        ],
        'minimal_fields': [
            'event_rule_id',
            'name',
            'order',
            'scope',
            'project_id'
        ],
        'change_query_keys': {
            'user_projects': 'project_id'
        },
        'ordering': ['order'],
        'indexes': [
            # 'event_rule_id',
            'order',
            'conditions_policy',
            'scope',
            'project_id',
            'domain_id'
        ]
    }



import functools
from typing import List
from spaceone.api.monitoring.v1 import escalation_policy_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils

from spaceone.monitoring.model.escalation_policy_model import EscalationPolicy, EscalationRule

__all__ = ['EscalationPolicyInfo', 'EscalationPoliciesInfo']


def EscalationRulesInfo(vos: List[EscalationRule]):
    if vos:
        rules = []
        for vo in vos:
            info = {
                'notification_level': vo.notification_level,
                'escalate_minutes': vo.escalate_minutes
            }
            rule = escalation_policy_pb2.EscalationPolicyRule(**info)
            rules.append(rule)

        return rules
    else:
        return None


def EscalationPolicyInfo(escalation_policy_vo: EscalationPolicy, minimal=False):
    info = {
        'escalation_policy_id': escalation_policy_vo.escalation_policy_id,
        'name': escalation_policy_vo.name,
        'is_default': escalation_policy_vo.is_default
    }

    if not minimal:
        info.update({
            'rules': EscalationRulesInfo(escalation_policy_vo.rules),
            'repeat_count': escalation_policy_vo.repeat_count,
            'finish_condition': escalation_policy_vo.finish_condition,
            'tags': change_struct_type(escalation_policy_vo.tags),
            'scope': escalation_policy_vo.scope,
            'project_id': escalation_policy_vo.project_id,
            'domain_id': escalation_policy_vo.domain_id,
            'created_at': utils.datetime_to_iso8601(escalation_policy_vo.created_at)
        })

    return escalation_policy_pb2.EscalationPolicyInfo(**info)


def EscalationPoliciesInfo(escalation_policy_vos, total_count, **kwargs):
    return escalation_policy_pb2.EscalationPoliciesInfo(results=list(
        map(functools.partial(EscalationPolicyInfo, **kwargs), escalation_policy_vos)), total_count=total_count)



import logging
from google.protobuf.json_format import MessageToDict
from spaceone.core.connector import BaseConnector
from spaceone.core import pygrpc
from spaceone.core.utils import parse_endpoint

__all__ = ['DataSourcePluginConnector']

_LOGGER = logging.getLogger(__name__)


class DataSourcePluginConnector(BaseConnector):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.client = None

    def initialize(self, endpoint):
        static_endpoint = self.config.get('endpoint')

        if static_endpoint:
            endpoint = static_endpoint

        e = parse_endpoint(endpoint)
        self.client = pygrpc.client(endpoint=f'{e.get("hostname")}:{e.get("port")}', version='plugin')

    def init(self, options):
        response = self.client.DataSource.init({
            'options': options,
        }, metadata=self.transaction.get_connection_meta())

        return self._change_message(response)

    def verify(self, options, secret_data, schema=None):
        params = {
            'options': options,
            'secret_data': secret_data
        }

        if schema:
            params.update({'schema': schema})

        self.client.DataSource.verify(params, metadata=self.transaction.get_connection_meta())

    def list_metrics(self, schema, options, secret_data, query):
        params = {
            'options': options,
            'secret_data': secret_data,
            'query': query
        }

        if schema:
            params.update({'schema': schema})

        response = self.client.Metric.list(params, metadata=self.transaction.get_connection_meta())
        return self._change_message(response)

    def get_metric_data(self, params):
        response = self.client.Metric.get_data(params, metadata=self.transaction.get_connection_meta())
        return self._change_message(response)

    def list_logs(self, schema, options, secret_data, query, keyword, start, end, sort, limit):
        params = {
            'options': options,
            'secret_data': secret_data,
            'query': query,
            'start': start,
            'end': end
        }

        if keyword:
            params.update({'keyword': keyword})

        if schema:
            params.update({'schema': schema})

        if sort:
            params.update({'sort': sort})

        if limit:
            params.update({'limit': limit})

        responses = self.client.Log.list(params, metadata=self.transaction.get_connection_meta())

        for response in responses:
            yield self._change_message(response)

    @staticmethod
    def _change_message(message):
        return MessageToDict(message, preserving_proto_field_name=True)



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel


class PluginInfo(EmbeddedDocument):
    plugin_id = StringField(max_length=40)
    version = StringField(max_length=255)
    options = DictField(default={})
    metadata = DictField(default={})
    upgrade_mode = StringField(max_length=255, choices=('AUTO', 'MANUAL'), default='AUTO')

    def to_dict(self):
        return dict(self.to_mongo())


class Webhook(MongoModel):
    webhook_id = StringField(max_length=40, generate_id='webhook', unique=True)
    name = StringField(max_length=255, unique_with=['project_id', 'domain_id'])
    state = StringField(max_length=20, default='ENABLED', choices=('ENABLED', 'DISABLED'))
    access_key = StringField(max_length=255)
    webhook_url = StringField(max_length=255)
    capability = DictField()
    plugin_info = EmbeddedDocumentField(PluginInfo, default=None, null=True)
    tags = DictField()
    project = ReferenceField('ProjectAlertConfig', reverse_delete_rule=CASCADE)
    project_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        'updatable_fields': [
            'name',
            'state',
            'access_key',
            'webhook_url',
            'capability',
            'plugin_info',
            'tags'
        ],
        'minimal_fields': [
            'webhook_id',
            'name',
            'state',
            'webhook_url',
            'project_id'
        ],
        'change_query_keys': {
            'user_projects': 'project_id'
        },
        'ordering': ['name'],
        'indexes': [
            # 'webhook_id',
            'state',
            'access_key',
            'project_id',
            'domain_id'
        ]
    }



import logging
import copy

from spaceone.core.service import *
from spaceone.core import utils, cache, config
from spaceone.monitoring.error.webhook import *
from spaceone.monitoring.model.event_model import Event
from spaceone.monitoring.model.alert_model import Alert
from spaceone.monitoring.model.webhook_model import Webhook
from spaceone.monitoring.model.project_alert_config_model import ProjectAlertConfig
from spaceone.monitoring.model.escalation_policy_model import EscalationPolicy
from spaceone.monitoring.manager.alert_manager import AlertManager
from spaceone.monitoring.manager.webhook_manager import WebhookManager
from spaceone.monitoring.manager.event_manager import EventManager
from spaceone.monitoring.manager.event_rule_manager import EventRuleManager
from spaceone.monitoring.manager.job_manager import JobManager
from spaceone.monitoring.manager.webhook_plugin_manager import WebhookPluginManager
from spaceone.monitoring.manager.project_alert_config_manager import ProjectAlertConfigManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler(exclude=['create'])
@authorization_handler(exclude=['create'])
@mutation_handler
@event_handler
class EventService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.resource = 'Event'
        self.event_mgr: EventManager = self.locator.get_manager('EventManager')
        self.webhook_mgr: WebhookManager = self.locator.get_manager('WebhookManager')

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['webhook_id', 'access_key', 'data'])
    def create(self, params):
        """Create event

        Args:
            params (dict): {
                'webhook_id': 'str',
                'access_key': 'str',
                'data': 'str'
            }

        Returns:
            event_vo (object)
        """

        webhook_data = self._get_webhook_data(params['webhook_id'])

        self._check_access_key(params['access_key'], webhook_data['access_key'])
        self._check_webhook_state(webhook_data)

        try:
            webhook_plugin_mgr: WebhookPluginManager = self.locator.get_manager('WebhookPluginManager')
            endpoint, updated_version = webhook_plugin_mgr.get_webhook_plugin_endpoint({
                'plugin_id': webhook_data['plugin_id'],
                'version': webhook_data['plugin_version'],
                'upgrade_mode': webhook_data['plugin_upgrade_mode']
            }, webhook_data['domain_id'])

            if updated_version:
                _LOGGER.debug(f'[create] upgrade plugin version: {webhook_data["plugin_version"]} -> {updated_version}')
                webhook_vo: Webhook = self.webhook_mgr.get_webhook(webhook_data['webhook_id'],
                                                                   webhook_data['domain_id'])
                webhook_plugin_mgr.upgrade_webhook_plugin_version(webhook_vo, endpoint, updated_version)
                cache.delete(f'webhook-data:{webhook_data["webhook_id"]}')

            webhook_plugin_mgr.initialize(endpoint)
            response = webhook_plugin_mgr.parse_event(webhook_data['plugin_options'], params['data'])

        except Exception as e:
            if not isinstance(e, ERROR_BASE):
                e = ERROR_UNKNOWN(message=str(e))

            _LOGGER.error(f'[create] Event parsing failed: {e.message}', exc_info=True)
            response = self._create_error_event(webhook_data['name'], e.message)

        for event_data in response.get('results', []):
            # TODO: Check event data using schematics

            _LOGGER.debug(f'[Event.create] event_data: {event_data}')
            self._create_event(event_data, params['data'], webhook_data)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['event_id', 'domain_id'])
    def get(self, params):
        """ Get event

        Args:
            params (dict): {
                'event_id': 'str',
                'domain_id': 'str',
                'only': 'list
            }

        Returns:
            event_vo (object)
        """

        return self.event_mgr.get_event(params['event_id'], params['domain_id'], params.get('only'))

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['domain_id'])
    @append_query_filter(['event_id', 'event_key', 'event_type', 'severity', 'resource_id', 'alert_id',
                          'webhook_id', 'project_id', 'domain_id', 'user_projects'])
    @append_keyword_filter(['event_id', 'title'])
    def list(self, params):
        """ List events

        Args:
            params (dict): {
                'event_id': 'str',
                'event_key': 'str',
                'event_type': 'str',
                'severity': 'str',
                'resource_id': 'str',
                'alert_id': 'str',
                'webhook_id': 'str',
                'project_id': 'str',
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)',
                'user_projects': 'list', // from meta
            }

        Returns:
            event_vos (object)
            total_count
        """

        query = params.get('query', {})
        return self.event_mgr.list_events(query)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['query', 'domain_id'])
    @append_query_filter(['domain_id', 'user_projects'])
    @append_keyword_filter(['event_id', 'title'])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',
                'user_projects': 'list', // from meta
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get('query', {})
        return self.event_mgr.stat_events(query)

    @cache.cacheable(key='webhook-data:{webhook_id}', expire=300)
    def _get_webhook_data(self, webhook_id):
        webhook_vo: Webhook = self.webhook_mgr.get_webhook_by_id(webhook_id)
        return {
            'webhook_id': webhook_vo.webhook_id,
            'name': webhook_vo.name,
            'project_id': webhook_vo.project_id,
            'domain_id': webhook_vo.domain_id,
            'state': webhook_vo.state,
            'access_key': webhook_vo.access_key,
            'plugin_id': webhook_vo.plugin_info.plugin_id,
            'plugin_version': webhook_vo.plugin_info.version,
            'plugin_upgrade_mode': webhook_vo.plugin_info.upgrade_mode,
            'plugin_options': webhook_vo.plugin_info.options
        }

    @staticmethod
    def _check_access_key(request_access_key, webhook_access_key):
        if request_access_key != webhook_access_key:
            raise ERROR_PERMISSION_DENIED()

    @staticmethod
    def _check_webhook_state(webhook_data):
        if webhook_data['state'] == 'DISABLED':
            raise ERROR_WEBHOOK_STATE_DISABLED(webhook_id=webhook_data['webhook_id'])

    def _create_event(self, event_data, raw_data, webhook_data):
        event_data['raw_data'] = copy.deepcopy(raw_data)
        event_data['occurred_at'] = utils.iso8601_to_datetime(event_data.get('occurred_at'))
        event_data['webhook_id'] = webhook_data['webhook_id']
        event_data['project_id'] = webhook_data['project_id']
        event_data['domain_id'] = webhook_data['domain_id']
        event_data['severity'] = event_data.get('severity', 'NONE')
        event_data['account'] = event_data.get('account', '')
        event_data['provider'] = event_data.get('provider', '')

        event_rule_mgr: EventRuleManager = self.locator.get_manager('EventRuleManager')

        # Change event data by event rule
        event_data = event_rule_mgr.change_event_data(event_data, webhook_data['project_id'], webhook_data['domain_id'])

        event_vo: Event = self.event_mgr.get_event_by_key(event_data['event_key'], event_data['domain_id'],
                                                          event_data['project_id'])

        if event_vo and event_vo.alert.state != 'RESOLVED':
            # Resolve alert when receiving recovery event
            if event_data['event_type'] == 'RECOVERY':
                self._update_alert_state(event_vo.alert)

            event_data['alert_id'] = event_vo.alert_id
            event_data['alert'] = event_vo.alert
        else:
            # Skip health event
            if event_data['event_type'] == 'RECOVERY':
                _LOGGER.debug(f'[_create_event] Skip health event: {event_data.get("title")} (event_type = RECOVERY)')
                return None

            # Create new alert
            _LOGGER.debug(f'[_create_event] Create new alert: {event_data.get("title")} '
                          f'(event_type = {event_data["event_type"]})')
            alert_vo = self._create_alert(event_data)

            event_data['alert_id'] = alert_vo.alert_id
            event_data['alert'] = alert_vo

        self.event_mgr.create_event(event_data)

    def _create_alert(self, event_data):
        alert_mgr: AlertManager = self.locator.get_manager('AlertManager')

        alert_data = copy.deepcopy(event_data)

        if 'urgency' in event_data:
            alert_data['urgency'] = event_data['urgency']
        else:
            alert_data['urgency'] = self._get_urgency_from_severity(event_data['severity'])

        escalation_policy_id, escalation_ttl = self._get_escalation_policy_info(event_data['project_id'],
                                                                                event_data['domain_id'])

        alert_data['escalation_policy_id'] = escalation_policy_id

        if event_data.get('no_notification', False):
            alert_data['escalation_ttl'] = 0
        else:
            alert_data['escalation_ttl'] = escalation_ttl + 1

        alert_data['triggered_by'] = alert_data['webhook_id']

        if event_data.get('event_type', 'ERROR') == 'ERROR':
            alert_data['state'] = 'ERROR'

        alert_vo = alert_mgr.create_alert(alert_data)

        self._create_notification(alert_vo, 'create_alert_notification')

        return alert_vo

    @staticmethod
    def _get_urgency_from_severity(severity):
        if severity in ['CRITICAL', 'ERROR', 'NOT_AVAILABLE']:
            return 'HIGH'
        else:
            return 'LOW'

    @cache.cacheable(key='escalation-policy-info:{domain_id}:{project_id}', expire=300)
    def _get_escalation_policy_info(self, project_id, domain_id):
        project_alert_config_vo: ProjectAlertConfig = self._get_project_alert_config(project_id, domain_id)
        escalation_policy_vo: EscalationPolicy = project_alert_config_vo.escalation_policy

        return escalation_policy_vo.escalation_policy_id, escalation_policy_vo.repeat_count

    def _update_alert_state(self, alert_vo: Alert):
        if self._is_auto_recovery(alert_vo.project_id, alert_vo.domain_id) and alert_vo.state != 'RESOLVED':
            alert_mgr: AlertManager = self.locator.get_manager('AlertManager')
            alert_mgr.update_alert_by_vo({'state': 'RESOLVED'}, alert_vo)

            self._create_notification(alert_vo, 'create_resolved_notification')

    @cache.cacheable(key='auto-recovery:{domain_id}:{project_id}', expire=300)
    def _is_auto_recovery(self, project_id, domain_id):
        project_alert_config_vo: ProjectAlertConfig = self._get_project_alert_config(project_id, domain_id)
        return project_alert_config_vo.options.recovery_mode == 'AUTO'

    def _get_project_alert_config(self, project_id, domain_id):
        project_alert_config_mgr: ProjectAlertConfigManager = self.locator.get_manager('ProjectAlertConfigManager')
        return project_alert_config_mgr.get_project_alert_config(project_id, domain_id)

    def _get_notification_urgency(self, project_id, domain_id):
        project_alert_config_vo: ProjectAlertConfig = self._get_project_alert_config(project_id, domain_id)
        return project_alert_config_vo.options.notification_urgency

    def _create_notification(self, alert_vo: Alert, method):
        # if alert_vo.state != 'ERROR':
        self._set_transaction_token()

        job_mgr: JobManager = self.locator.get_manager('JobManager')
        job_mgr.push_task(
            'monitoring_alert_notification_from_webhook',
            'JobService',
            method,
            {
                'alert_id': alert_vo.alert_id,
                'domain_id': alert_vo.domain_id
            }
        )

    def _set_transaction_token(self):
        self.transaction.set_meta('token', config.get_global('TOKEN'))

    @staticmethod
    def _create_error_event(webhook_name, error_message):
        response = {
            'results': [
                {
                    'event_key': utils.generate_id('error'),
                    'event_type': 'ERROR',
                    'title': f'Webhook Event Parsing Error - {webhook_name}',
                    'description': error_message,
                    'severity': 'CRITICAL'
                }
            ]
        }

        return response



import logging
from datetime import datetime, timedelta

from spaceone.core import config
from spaceone.core.manager import BaseManager
from spaceone.monitoring.model.event_model import Event

_LOGGER = logging.getLogger(__name__)


class EventManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.event_model: Event = self.locator.get_model('Event')

    def create_event(self, params):
        def _rollback(event_vo):
            _LOGGER.info(f'[create_event._rollback] '
                         f'Delete event : {event_vo.event_id}')
            event_vo.delete()

        event_vo: Event = self.event_model.create(params)
        self.transaction.add_rollback(_rollback, event_vo)

        return event_vo

    def update_event(self, params):
        event_vo: Event = self.get_event(params['event_id'], params['domain_id'])
        return self.update_event_by_vo(params, event_vo)

    def update_event_by_vo(self, params, event_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[update_event_by_vo._rollback] Revert Data : '
                         f'{old_data["event_id"]}')
            event_vo.update(old_data)

        self.transaction.add_rollback(_rollback, event_vo.to_dict())

        return event_vo.update(params)

    def delete_event(self, event_id, domain_id):
        event_vo: Event = self.get_event(event_id, domain_id)
        event_vo.delete()

    def get_event(self, event_id, domain_id, only=None):
        return self.event_model.get(event_id=event_id, domain_id=domain_id, only=only)

    def filter_events(self, **conditions):
        return self.event_model.filter(**conditions)

    def list_events(self, query={}):
        return self.event_model.query(**query)

    def stat_events(self, query):
        return self.event_model.stat(**query)

    def get_event_by_key(self, event_key, domain_id, project_id):
        same_event_time = config.get_global('SAME_EVENT_TIME', 600)
        same_event_datetime = datetime.utcnow() - timedelta(seconds=same_event_time)

        query = {
            'filter': [
                {
                    'k': 'event_key',
                    'v': event_key,
                    'o': 'eq'
                },
                {
                    'k': 'domain_id',
                    'v': domain_id,
                    'o': 'eq'
                },
                {
                    'k': 'project_id',
                    'v': project_id,
                    'o': 'eq'
                },
                {
                    'k': 'event_type',
                    'v': 'RECOVERY',
                    'o': 'not'
                },
                {
                    'k': 'created_at',
                    'v': same_event_datetime,
                    'o': 'gte'
                }
            ],
            'sort': {
                'key': 'created_at',
                'desc': True
            }
        }

        event_vos, total_count = self.list_events(query)
        if event_vos.count() > 0:
            return event_vos[0]
        else:
            return None



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel
from pydantic import BaseModel, Field
from typing import Union
from datetime import datetime


class EventResource(EmbeddedDocument):
    resource_id = StringField(default=None, null=True)
    resource_type = StringField(default=None, null=True)
    name = StringField(default=None, null=True)


class Event(MongoModel):
    event_id = StringField(max_length=40, generate_id='event', unique=True)
    event_key = StringField()
    event_type = StringField(max_length=20, default='ALERT', choices=('ALERT', 'RECOVERY', 'ERROR'))
    title = StringField()
    description = StringField(default=None, null=True)
    severity = StringField(max_length=20, default='NONE', choices=('CRITICAL', 'ERROR', 'WARNING', 'INFO',
                                                                   'NOT_AVAILABLE', 'NONE'))
    rule = StringField(default=None, null=True)
    image_url = StringField(default=None, null=True)
    resource = EmbeddedDocumentField(EventResource, default=None, null=True)
    provider = StringField(default=None, null=True)
    account = StringField(default=None, null=True)
    raw_data = DictField()
    additional_info = DictField()
    alert = ReferenceField('Alert', reverse_delete_rule=CASCADE)
    alert_id = StringField(max_length=40)
    webhook_id = StringField(max_length=40)
    project_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)
    occurred_at = DateTimeField(default=None, null=True)

    meta = {
        'updatable_fields': [
            'alert_id',
            'project_id',
            'alert',
            'alert_id'
        ],
        'minimal_fields': [
            'event_id',
            'event_key',
            'event_type',
            'title',
            'severity',
            'alert_id'
        ],
        'change_query_keys': {
            'user_projects': 'project_id',
            'resource_id': 'resource.resource_id'
        },
        'ordering': [
            '-created_at'
        ],
        'indexes': [
            # 'event_id',
            'event_key',
            'event_type',
            'severity',
            'resource.resource_id',
            'resource.resource_type',
            'resource.name',
            'provider',
            'account',
            'alert',
            'alert_id',
            'webhook_id',
            'project_id',
            'domain_id',
            'created_at',
            'occurred_at'
        ]
    }

# Pydantic Model


class CreateEventResource(BaseModel):
    resource_id: Union[str, None] = Field(None)
    resource_type: Union[str, None] = Field(None)
    name: Union[str, None] = Field(None)


class CreateEvent(BaseModel):
    event_key: Union[str, None] = Field(None)
    event_type: Union[str, None] = Field(None)
    title: Union[str, None] = Field(None)
    description: Union[str, None] = Field(None)
    severity: Union[str, None] = Field(None)
    rule: Union[str, None] = Field(None)
    image_url: Union[str, None] = Field(None)
    resource: Union[CreateEventResource, None] = Field(None)
    additional_info: Union[dict, None] = Field(None)
    project_id: Union[str, None] = Field(None)
    occurred_at: Union[datetime, None] = Field(None)



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel
from spaceone.monitoring.model.escalation_policy_model import EscalationPolicy


class AlertOptions(EmbeddedDocument):
    notification_urgency = StringField(max_length=20, default='ALL', choices=('ALL', 'HIGH_ONLY'))
    recovery_mode = StringField(max_length=20, default='MANUAL', choices=('MANUAL', 'AUTO'))

    def to_dict(self):
        return self.to_mongo()


class ProjectAlertConfig(MongoModel):
    project_id = StringField(max_length=40, unique=True)
    options = EmbeddedDocumentField(AlertOptions, default=AlertOptions)
    escalation_policy = ReferenceField('EscalationPolicy', reverse_delete_rule=DENY)
    escalation_policy_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        'updatable_fields': [
            'options',
            'escalation_policy',
            'escalation_policy_id'
        ],
        'minimal_fields': [
            'project_id',
            'options'
        ],
        'change_query_keys': {
            'user_projects': 'project_id'
        },
        'reference_query_keys': {
            'escalation_policy': EscalationPolicy
        },
        'indexes': [
            # 'project_id',
            'escalation_policy',
            'escalation_policy_id',
            'domain_id'
        ]
    }



import logging

from spaceone.core import cache
from spaceone.core import config
from spaceone.core.manager import BaseManager
from spaceone.core.connector.space_connector import SpaceConnector

_LOGGER = logging.getLogger(__name__)

_GET_RESOURCE_METHODS = {
    'identity.Project': 'get_project',
    'identity.ServiceAccount': 'get_service_account',
}


class IdentityManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.identity_connector: SpaceConnector = self.locator.get_connector('SpaceConnector', service='identity',
                                                                             token=config.get_global('TOKEN'))

    def get_domain(self, domain_id):
        return self.identity_connector.dispatch('Domain.get', {'domain_id': domain_id})

    def get_user(self, user_id, domain_id):
        return self.identity_connector.dispatch('User.get', {'user_id': user_id, 'domain_id': domain_id})

    def get_project(self, project_id, domain_id):
        return self.identity_connector.dispatch('Project.get', {'project_id': project_id, 'domain_id': domain_id})

    def get_service_account(self, service_account_id, domain_id):
        return self.identity_connector.dispatch('ServiceAccount.get', {'service_account_id': service_account_id,
                                                                       'domain_id': domain_id})

    def get_resource(self, resource_type, resource_id, domain_id):
        if resource_type == 'identity.Project':
            return self.get_project(resource_id, domain_id)
        elif resource_type == 'identity.ServiceAccount':
            return self.get_service_account(resource_id, domain_id)

    def list_service_accounts(self, query, domain_id):
        return self.identity_connector.dispatch('ServiceAccount.list', {'query': query, 'domain_id': domain_id})



import logging

from spaceone.core.manager import BaseManager
from spaceone.monitoring.model.maintenance_window_model import MaintenanceWindow

_LOGGER = logging.getLogger(__name__)


class MaintenanceWindowManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.maintenance_window_model: MaintenanceWindow = self.locator.get_model('MaintenanceWindow')

    def create_maintenance_window(self, params):
        def _rollback(maintenance_window_vo):
            _LOGGER.info(f'[create_maintenance_window._rollback] '
                         f'Delete maintenance_window : {maintenance_window_vo.title} '
                         f'({maintenance_window_vo.maintenance_window_id})')
            maintenance_window_vo.delete()

        maintenance_window_vo: MaintenanceWindow = self.maintenance_window_model.create(params)
        self.transaction.add_rollback(_rollback, maintenance_window_vo)

        return maintenance_window_vo

    def update_maintenance_window(self, params):
        maintenance_window_vo: MaintenanceWindow = self.get_maintenance_window(params['maintenance_window_id'],
                                                                               params['domain_id'])
        return self.update_maintenance_window_by_vo(params, maintenance_window_vo)

    def update_maintenance_window_by_vo(self, params, maintenance_window_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[update_maintenance_window_by_vo._rollback] Revert Data : '
                         f'{old_data["maintenance_window_id"]}')
            maintenance_window_vo.update(old_data)

        self.transaction.add_rollback(_rollback, maintenance_window_vo.to_dict())

        return maintenance_window_vo.update(params)

    def close_maintenance_window(self, maintenance_window_id, domain_id):
        maintenance_window_vo: MaintenanceWindow = self.get_maintenance_window(maintenance_window_id, domain_id)

        return self.update_maintenance_window_by_vo({'state': 'CLOSED'}, maintenance_window_vo)

    def get_maintenance_window(self, maintenance_window_id, domain_id, only=None):
        return self.maintenance_window_model.get(maintenance_window_id=maintenance_window_id,
                                                 domain_id=domain_id, only=only)

    def list_open_maintenance_windows(self):
        return self.maintenance_window_model.filter(state='OPEN')

    def list_maintenance_windows(self, query={}):
        return self.maintenance_window_model.query(**query)

    def stat_maintenance_windows(self, query):
        return self.maintenance_window_model.stat(**query)



from spaceone.core.error import *


class ERROR_INVALID_ESCALATION_POLICY(ERROR_INVALID_ARGUMENT):
    _message = 'Escalation policy is invalid. (escalation_policy_id = {escalation_policy_id})'


class ERROR_ALERT_FEATURE_IS_NOT_ACTIVATED(ERROR_INVALID_ARGUMENT):
    _message = 'The alert feature is not activated in the project. (project_id = {project_id})'



import logging
from typing import List
from datetime import datetime

from spaceone.core.service import *
from spaceone.monitoring.error.maintenance_window import *
from spaceone.monitoring.model.maintenance_window_model import MaintenanceWindow
from spaceone.monitoring.manager.project_alert_config_manager import ProjectAlertConfigManager
from spaceone.monitoring.manager.maintenance_window_manager import MaintenanceWindowManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class MaintenanceWindowService(BaseService):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.maintenance_window_mgr: MaintenanceWindowManager = self.locator.get_manager('MaintenanceWindowManager')

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['title', 'projects', 'start_time', 'end_time', 'domain_id'])
    @change_timestamp_value(['start_time', 'end_time'], timestamp_format='iso8601')
    def create(self, params):
        """Create maintenance window

        Args:
            params (dict): {
                'title': 'str',
                'projects': 'list',
                'start_time': 'datetime',
                'end_time': 'datetime',
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            maintenance_window_vo (object)
        """

        domain_id = params['domain_id']
        projects = params['projects']

        params['created_by'] = self.transaction.get_meta('user_id')

        project_alert_config_mgr: ProjectAlertConfigManager = self.locator.get_manager('ProjectAlertConfigManager')

        # TODO: Check projects and user permissions

        return self.maintenance_window_mgr.create_maintenance_window(params)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['maintenance_window_id', 'domain_id'])
    @change_timestamp_value(['start_time', 'end_time'], timestamp_format='iso8601')
    def update(self, params):
        """Update maintenance window

        Args:
            params (dict): {
                'maintenance_window_id': 'str',
                'title': 'str',
                'projects': 'list',
                'start_time': 'datetime',
                'end_time': 'datetime',
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            maintenance_window_vo (object)
        """

        maintenance_window_id = params['maintenance_window_id']
        domain_id = params['domain_id']
        projects = params.get('projects')

        maintenance_window_vo = self.maintenance_window_mgr.get_maintenance_window(maintenance_window_id, domain_id)

        if maintenance_window_vo.state == 'CLOSED':
            raise ERROR_CLOSED_MAINTENANCE_WINDOW(maintenance_window_id=maintenance_window_id)

        if projects:
            project_alert_config_mgr: ProjectAlertConfigManager = self.locator.get_manager('ProjectAlertConfigManager')

            # TODO: Check projects and user permissions

        return self.maintenance_window_mgr.update_maintenance_window_by_vo(params, maintenance_window_vo)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['maintenance_window_id', 'domain_id'])
    def close(self, params):
        """Delete maintenance window

        Args:
            params (dict): {
                'maintenance_window_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """

        return self.maintenance_window_mgr.close_maintenance_window(params['maintenance_window_id'],
                                                                    params['domain_id'])

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['maintenance_window_id', 'domain_id'])
    def get(self, params):
        """ Get maintenance window

        Args:
            params (dict): {
                'maintenance_window_id': 'str',
                'domain_id': 'str',
                'only': 'list
            }

        Returns:
            maintenance_window_vo (object)
        """

        return self.maintenance_window_mgr.get_maintenance_window(params['maintenance_window_id'],
                                                                  params['domain_id'], params.get('only'))

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['domain_id'])
    @append_query_filter(['maintenance_window_id', 'title', 'state', 'project_id', 'created_by', 'domain_id',
                          'user_projects'])
    @append_keyword_filter(['maintenance_window_id', 'title'])
    def list(self, params):
        """ List maintenance windows

        Args:
            params (dict): {
                'maintenance_window_id': 'str',
                'title': 'str',
                'state': 'str',
                'project_id': 'str',
                'created_by': 'str',
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)',
                'user_projects': 'list', // from meta
            }

        Returns:
            maintenance_window_vos (object)
            total_count
        """

        query = params.get('query', {})
        return self.maintenance_window_mgr.list_maintenance_windows(query)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['query', 'domain_id'])
    @append_query_filter(['domain_id', 'user_projects'])
    @append_keyword_filter(['maintenance_window_id', 'title'])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',
                'user_projects': 'list', // from meta
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get('query', {})
        return self.maintenance_window_mgr.stat_maintenance_windows(query)

    @transaction(append_meta={'authorization.scope': 'SYSTEM'})
    def close_maintenance_window(self, params):
        """ Close out of time maintenance window

        Args:
            params (dict): {}

        Returns:
            None
        """
        maintenance_window_mgr: MaintenanceWindowManager = self.locator.get_manager('MaintenanceWindowManager')
        maintenance_window_vos: List[MaintenanceWindow] = maintenance_window_mgr.list_open_maintenance_windows()

        current_time = datetime.utcnow()

        for maintenance_window_vo in maintenance_window_vos:
            if current_time > maintenance_window_vo.end_time:
                _LOGGER.debug(f'[close_maintenance_window] Close out of time maintenance window '
                              f'({maintenance_window_vo.maintenance_window_id}): '
                              f'Current Time({current_time}) > End Time({maintenance_window_vo.end_time})')
                maintenance_window_mgr.update_maintenance_window_by_vo({'state': 'CLOSED'}, maintenance_window_vo)



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel


class EscalationRule(EmbeddedDocument):
    notification_level = StringField(max_length=20, default='ALL')
    escalate_minutes = IntField(default=0, min_value=0)

    def to_dict(self):
        return self.to_mongo()


class EscalationPolicy(MongoModel):
    escalation_policy_id = StringField(max_length=40, generate_id='ep', unique=True)
    name = StringField(max_length=255, unique_with='domain_id')
    is_default = BooleanField(default=False)
    rules = ListField(EmbeddedDocumentField(EscalationRule))
    repeat_count = IntField(default=0, min_value=0)
    finish_condition = StringField(max_length=20, default='ACKNOWLEDGED', choices=('ACKNOWLEDGED', 'RESOLVED'))
    tags = DictField()
    scope = StringField(max_length=20, choices=('DOMAIN', 'PROJECT'))
    project_id = StringField(max_length=40, default=None, null=True)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        'updatable_fields': [
            'name',
            'is_default',
            'rules',
            'repeat_count',
            'finish_condition',
            'tags'
        ],
        'minimal_fields': [
            'escalation_policy_id',
            'name',
            'is_default'
        ],
        'change_query_keys': {
            'user_projects': 'project_id'
        },
        'ordering': ['name'],
        'indexes': [
            # 'escalation_policy_id',
            'is_default',
            'finish_condition',
            'scope',
            'project_id',
            'domain_id'
        ]
    }



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel


class PluginInfo(EmbeddedDocument):
    plugin_id = StringField(max_length=40)
    version = StringField(max_length=255)
    options = DictField(default={})
    metadata = DictField(default={})
    secret_id = StringField(max_length=40, default=None, null=True)
    provider = StringField(max_length=40, default=None, null=True)
    upgrade_mode = StringField(max_length=255, choices=('AUTO', 'MANUAL'), default='AUTO')

    def to_dict(self):
        return dict(self.to_mongo())


class DataSource(MongoModel):
    data_source_id = StringField(max_length=40, generate_id='ds', unique=True)
    name = StringField(max_length=255, unique_with='domain_id')
    state = StringField(max_length=20, default='ENABLED', choices=('ENABLED', 'DISABLED'))
    monitoring_type = StringField(max_length=20, choices=('METRIC', 'LOG'))
    provider = StringField(max_length=40, default=None, null=True)
    capability = DictField()
    plugin_info = EmbeddedDocumentField(PluginInfo, default=None, null=True)
    tags = DictField()
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        'updatable_fields': [
            'name',
            'state',
            'capability',
            'plugin_info',
            'tags'
        ],
        'minimal_fields': [
            'data_source_id',
            'name',
            'state',
            'monitoring_type',
            'provider'
        ],
        'ordering': [
            'name'
        ],
        'indexes': [
            # 'data_source_id',
            'state',
            'monitoring_type',
            'provider',
            'domain_id',
        ]
    }



import logging

from spaceone.core.manager import BaseManager
from spaceone.core.connector.space_connector import SpaceConnector
from spaceone.monitoring.error import *

_LOGGER = logging.getLogger(__name__)


class SecretManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.secret_connector: SpaceConnector = self.locator.get_connector('SpaceConnector', service='secret')

    def get_secret_from_resource(self, resource, data_source_vo, domain_id):
        secret = {}

        if data_source_plugin_info := data_source_vo.plugin_info:
            if secret_id := data_source_plugin_info.secret_id:
                secret = self.list_secrets_from_query({'secret_id': secret_id}, domain_id)[0]

        if not secret:
            resource_secrets = []
            for collection_info in resource.get('collection_info', []):
                if secret_id := collection_info.get('secret_id'):
                    resource_secrets.append(secret_id)

            if data_source_vo.capability.get('use_resource_secret', False):
                secret_filter = {
                    'provider': data_source_vo.plugin_info.provider,
                    'supported_schema': data_source_vo.capability.get('supported_schema', []),
                    'secrets': resource_secrets
                }
                secret = self.list_secrets_from_query(secret_filter, domain_id)[0]

        return secret

    def list_secrets(self, query, domain_id):
        return self.secret_connector.dispatch('Secret.list', {'query': query, 'domain_id': domain_id})

    def get_secret_data(self, secret_id, domain_id):
        response = self.secret_connector.dispatch('Secret.get_data', {'secret_id': secret_id, 'domain_id': domain_id})
        return response['data']

    def list_secrets_from_query(self, secret_filter, domain_id, **kwargs):
        secret_query = self._make_query(**secret_filter)
        response = self.list_secrets(secret_query, domain_id)

        if response.get('total_count', 0) == 0:
            resource_id = kwargs.get('resource_id')
            raise ERROR_RESOURCE_SECRETS_NOT_EXISTS(resource_id=resource_id)

        return response.get('results', [])

    def get_secret_data_from_plugin(self, plugin_info, capability, domain_id):
        plugin_id = plugin_info['plugin_id']
        secret_id = plugin_info.get('secret_id')
        provider = plugin_info.get('provider')

        use_resource_secret = capability.get('use_resource_secret', False)
        supported_schema = capability.get('supported_schema', [])

        self._check_plugin_secret(use_resource_secret, plugin_info)

        if use_resource_secret:
            secret_query = self._make_query(supported_schema=supported_schema, provider=provider)
        else:
            secret_query = self._make_query(supported_schema=supported_schema, secret_id=secret_id)

        response = self.list_secrets(secret_query, domain_id)

        if response.get('total_count', 0) == 0:
            if use_resource_secret:
                raise ERROR_SUPPORTED_SECRETS_NOT_EXISTS(plugin_id=plugin_id, provider=provider)
            else:
                raise ERROR_NOT_FOUND(key='plugin_info.secret_id', value=secret_id)

        result = response['results'][0]
        secret_id = result['secret_id']
        schema = result.get('schema')

        return self.get_secret_data(secret_id, domain_id), schema

    @staticmethod
    def _check_plugin_secret(use_resource_secret, plugin_info):
        secret_id = plugin_info.get('secret_id')
        provider = plugin_info.get('provider')

        if use_resource_secret:
            if provider is None:
                raise ERROR_REQUIRED_PARAMETER(key='plugin_info.provider')
        else:
            if secret_id is None:
                raise ERROR_REQUIRED_PARAMETER(key='plugin_info.secret_id')

    @staticmethod
    def _make_query(**secret_filter):
        supported_schema = secret_filter.get('supported_schema')
        secret_id = secret_filter.get('secret_id')
        service_account_id = secret_filter.get('service_account_id')
        project_id = secret_filter.get('project_id')
        provider = secret_filter.get('provider')
        secrets = secret_filter.get('secrets')

        query = {
            'filter': []
        }

        if supported_schema:
            query['filter'].append({
                'k': 'schema',
                'v': supported_schema,
                'o': 'in'
            })

        if secret_id:
            query['filter'].append({
                'k': 'secret_id',
                'v': secret_id,
                'o': 'eq'
            })

        if provider:
            query['filter'].append({
                'k': 'provider',
                'v': provider,
                'o': 'eq'
            })

        if service_account_id:
            query['filter'].append({
                'k': 'service_account_id',
                'v': service_account_id,
                'o': 'eq'
            })

        if project_id:
            query['filter'].append({
                'k': 'project_id',
                'v': project_id,
                'o': 'eq'
            })

        if secrets:
            query['filter'].append({
                'k': 'secret_id',
                'v': secrets,
                'o': 'in'
            })

        return query



import copy
import logging

from spaceone.core import cache
from spaceone.core.manager import BaseManager
from spaceone.monitoring.error.escalation_policy import *
from spaceone.monitoring.model.escalation_policy_model import EscalationPolicy
from spaceone.monitoring.conf.default_escalation_policy import DEFAULT_ESCALATION_POLICY

_LOGGER = logging.getLogger(__name__)


class EscalationPolicyManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.escalation_policy_model: EscalationPolicy = self.locator.get_model('EscalationPolicy')

    def create_escalation_policy(self, params):
        def _rollback(escalation_policy_vo: EscalationPolicy):
            _LOGGER.info(f'[create_escalation_policy._rollback] '
                         f'Delete escalation policy : {escalation_policy_vo.name} '
                         f'({escalation_policy_vo.escalation_policy_id})')
            escalation_policy_vo.delete()

        escalation_policy_vo: EscalationPolicy = self.escalation_policy_model.create(params)
        self.transaction.add_rollback(_rollback, escalation_policy_vo)

        return escalation_policy_vo

    def update_escalation_policy(self, params):
        escalation_policy_vo: EscalationPolicy = self.get_escalation_policy(params['escalation_policy_id'],
                                                                            params['domain_id'])
        return self.update_escalation_policy_by_vo(params, escalation_policy_vo)

    def update_escalation_policy_by_vo(self, params, escalation_policy_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[update_escalation_policy_by_vo._rollback] Revert Data : '
                         f'{old_data["escalation_policy_id"]}')
            escalation_policy_vo.update(old_data)

        self.transaction.add_rollback(_rollback, escalation_policy_vo.to_dict())

        updated_vo: EscalationPolicy = escalation_policy_vo.update(params)

        cache.delete(f'escalation-policy-condition:{updated_vo.domain_id}:{updated_vo.escalation_policy_id}')

        return updated_vo

    def set_default_escalation_policy(self, params, escalation_policy_vo):
        global_escalation_policy_vos = self.escalation_policy_model.filter(domain_id=params['domain_id'],
                                                                           scope='DOMAIN')

        for global_escalation_policy_vo in global_escalation_policy_vos:
            global_escalation_policy_vo.update({'is_default': False})

        return escalation_policy_vo.update({'is_default': True})

    def get_default_escalation_policy(self, domain_id):
        escalation_policy_vos = self.escalation_policy_model.filter(is_default=True, domain_id=domain_id)
        if escalation_policy_vos.count() > 0:
            return escalation_policy_vos[0]
        else:
            default_escalation_policy = copy.deepcopy(DEFAULT_ESCALATION_POLICY)
            default_escalation_policy['domain_id'] = domain_id
            return self.create_escalation_policy(default_escalation_policy)

    def delete_escalation_policy(self, escalation_policy_id, domain_id):
        escalation_policy_vo: EscalationPolicy = self.get_escalation_policy(escalation_policy_id, domain_id)

        if escalation_policy_vo.is_default:
            raise ERROR_DEFAULT_ESCALATION_POLICY_NOT_ALLOW_DELETION(escalation_policy_id=escalation_policy_id)

        cache.delete(f'escalation-policy-condition:{domain_id}:{escalation_policy_id}')

        escalation_policy_vo.delete()

    def get_escalation_policy(self, escalation_policy_id, domain_id, only=None):
        return self.escalation_policy_model.get(escalation_policy_id=escalation_policy_id,
                                                domain_id=domain_id, only=only)

    def list_escalation_policies(self, query={}):
        return self.escalation_policy_model.query(**query)

    def stat_escalation_policies(self, query):
        return self.escalation_policy_model.stat(**query)



import logging
from fastapi import Request, Body
from fastapi_utils.inferring_router import InferringRouter
from fastapi_utils.cbv import cbv
from spaceone.core.fastapi.api import BaseAPI, exception_handler
from spaceone.core.error import *
from spaceone.core import config, cache
from spaceone.monitoring.model import Alert
from spaceone.monitoring.service import AlertService
from spaceone.monitoring.manager import IdentityManager
from fastapi.responses import RedirectResponse

_LOGGER = logging.getLogger(__name__)

router = InferringRouter()


@cbv(router)
class Alert(BaseAPI):

    service = 'Alert'

    @router.get('/alert/{alert_id}/{access_key}/{state}')
    @exception_handler
    async def update_alert_state_get(self, alert_id: str, access_key: str, state: str):
        alert_info = self._update_alert_state(alert_id, access_key, state)
        domain_name = self._get_domain_name(alert_info['domain_id'])

        return self._make_redirect_response(alert_id, domain_name)

    @router.post('/alert/{alert_id}/{access_key}/{state}')
    async def update_alert_state_post(self, alert_id: str, access_key: str, state: str, request: Request):
        params, metadata = await self.parse_request(request)

        if params.get('code') != 'TIME_OUT':
            self._update_alert_state(alert_id, access_key, state)

            return {
                'detail': f'Alert({alert_id}) state was changed.'
            }
        else:
            raise ERROR_REQUEST_TIMEOUT()

    def _update_alert_state(self, alert_id, access_key, state):
        alert_service: AlertService = self.locator.get_service('AlertService')

        alert_vo: Alert = alert_service.update_state({
            'alert_id': alert_id,
            'access_key': access_key,
            'state': state
        })

        return {
            'alert_number': alert_vo.alert_number,
            'alert_id': alert_vo.alert_id,
            'title': alert_vo.title,
            'state': alert_vo.state,
            'assignee': alert_vo.assignee,
            'urgency': alert_vo.urgency,
            'domain_id': alert_vo.domain_id
        }

    @cache.cacheable(key='domain-name:{domain_id}', expire=3600)
    def _get_domain_name(self, domain_id):
        identity_mgr: IdentityManager = self.locator.get_manager('IdentityManager')
        domain_info = identity_mgr.get_domain(domain_id)
        return domain_info['name']

    @staticmethod
    def _make_redirect_response(alert_id, domain_name):
        console_domain = config.get_global('CONSOLE_DOMAIN')

        if console_domain.strip() != '':
            console_domain = console_domain.format(domain_name=domain_name)
            return RedirectResponse(f'{console_domain}/alert-manager/alert/{alert_id}')
        else:
            return None



import functools
from spaceone.api.monitoring.v1 import event_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.monitoring.model.event_model import Event

__all__ = ['EventInfo', 'EventsInfo']


def EventResourceInfo(vo):
    if vo:
        info = {
            'resource_id': vo.resource_id,
            'resource_type': vo.resource_type,
            'name': vo.name
        }

        return event_pb2.EventResource(**info)
    else:
        return None


def EventInfo(event_vo: Event, minimal=False):
    info = {
        'event_id': event_vo.event_id,
        'event_key': event_vo.event_key,
        'event_type': event_vo.event_type,
        'title': event_vo.title,
        'severity': event_vo.severity,
        'alert_id': event_vo.alert_id
    }

    if not minimal:
        info.update({
            'description': event_vo.description,
            'rule': event_vo.rule,
            'provider': event_vo.provider,
            'account': event_vo.account,
            'image_url': event_vo.image_url,
            'resource': EventResourceInfo(event_vo.resource),
            'raw_data': change_struct_type(event_vo.raw_data),
            'additional_info': change_struct_type(event_vo.additional_info),
            'alert_id': event_vo.alert_id,
            'webhook_id': event_vo.webhook_id,
            'project_id': event_vo.project_id,
            'domain_id': event_vo.domain_id,
            'created_at': utils.datetime_to_iso8601(event_vo.created_at),
            'occurred_at': utils.datetime_to_iso8601(event_vo.occurred_at)
        })

    return event_pb2.EventInfo(**info)


def EventsInfo(note_vos, total_count, **kwargs):
    return event_pb2.EventsInfo(results=list(
        map(functools.partial(EventInfo, **kwargs), note_vos)), total_count=total_count)



import functools
from spaceone.api.monitoring.v1 import webhook_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.core import config
from spaceone.monitoring.model.webhook_model import Webhook

__all__ = ['WebhookInfo', 'WebhooksInfo']


def PluginInfo(vo):
    if vo:
        info = {
            'plugin_id': vo.plugin_id,
            'version': vo.version,
            'options': change_struct_type(vo.options),
            'metadata': change_struct_type(vo.metadata),
            'upgrade_mode': vo.upgrade_mode
        }

        return webhook_pb2.WebhookPluginInfo(**info)
    else:
        return None


def WebhookInfo(webhook_vo: Webhook, minimal=False):
    if webhook_vo.webhook_url:
        webhook_url = f'{config.get_global("WEBHOOK_DOMAIN")}{webhook_vo.webhook_url}'
    else:
        webhook_url = None

    info = {
        'webhook_id': webhook_vo.webhook_id,
        'name': webhook_vo.name,
        'state': webhook_vo.state,
        'webhook_url': webhook_url,
        'project_id': webhook_vo.project_id
    }

    if not minimal:
        info.update({
            'access_key': webhook_vo.access_key,
            'capability': change_struct_type(webhook_vo.capability),
            'plugin_info': PluginInfo(webhook_vo.plugin_info),
            'domain_id': webhook_vo.domain_id,
            'created_at': utils.datetime_to_iso8601(webhook_vo.created_at)
        })

    return webhook_pb2.WebhookInfo(**info)


def WebhooksInfo(webhook_vos, total_count, **kwargs):
    return webhook_pb2.WebhooksInfo(results=list(
        map(functools.partial(WebhookInfo, **kwargs), webhook_vos)), total_count=total_count)



import functools
from typing import List
from spaceone.api.monitoring.v1 import event_rule_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils

from spaceone.monitoring.model.event_rule_model import EventRule, EventRuleCondition, EventRuleOptions

__all__ = ['EventRuleInfo', 'EventRulesInfo']


def EventRuleConditionsInfo(condition_vos: List[EventRuleCondition]):
    if condition_vos is None:
        condition_vos = []

    conditions_info = []

    for vo in condition_vos:
        info = {
            'key': vo.key,
            'value': vo.value,
            'operator': vo.operator
        }

        conditions_info.append(event_rule_pb2.EventRuleCondition(**info))

    return conditions_info


def EventRuleActionResponderInfo(responders_data):
    if responders_data is None:
        responders_data = []

    responders_info = []

    for responder in responders_data:
        info = {
            'resource_type': responder.get('resource_type'),
            'resource_id': responder.get('resource_id')
        }

        responders_info.append(event_rule_pb2.EventRuleActionResponder(**info))

    return responders_info


def EventRuleActionsInfo(actions_data):
    if actions_data is None:
        return None
    else:
        info = {}

        for key, value in actions_data.items():
            if key == 'add_responder':
                info[key] = EventRuleActionResponderInfo(value)
            elif key == 'add_additional_info':
                info[key] = change_struct_type(value)
            else:
                info[key] = value

        return event_rule_pb2.EventRuleActions(**info)


def EventRuleOptionsInfo(vo: EventRuleOptions):
    if vo is None:
        return None
    else:
        info = {
            'stop_processing': vo.stop_processing
        }

        return event_rule_pb2.EventRuleOptions(**info)


def EventRuleInfo(event_rule_vo: EventRule, minimal=False):
    info = {
        'event_rule_id': event_rule_vo.event_rule_id,
        'name': event_rule_vo.name,
        'order': event_rule_vo.order,
        'scope': event_rule_vo.scope,
        'project_id': event_rule_vo.project_id,
    }

    if not minimal:
        info.update({
            'conditions': EventRuleConditionsInfo(event_rule_vo.conditions),
            'conditions_policy': event_rule_vo.conditions_policy,
            'actions': EventRuleActionsInfo(event_rule_vo.actions),
            'options': EventRuleOptionsInfo(event_rule_vo.options),
            'tags': change_struct_type(event_rule_vo.tags),
            'domain_id': event_rule_vo.domain_id,
            'created_at': utils.datetime_to_iso8601(event_rule_vo.created_at)
        })

    return event_rule_pb2.EventRuleInfo(**info)


def EventRulesInfo(event_rule_vos, total_count, **kwargs):
    return event_rule_pb2.EventRulesInfo(results=list(
        map(functools.partial(EventRuleInfo, **kwargs), event_rule_vos)), total_count=total_count)



from spaceone.api.monitoring.v1 import event_rule_pb2, event_rule_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class EventRule(BaseAPI, event_rule_pb2_grpc.EventRuleServicer):

    pb2 = event_rule_pb2
    pb2_grpc = event_rule_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('EventRuleService', metadata) as event_rule_service:
            return self.locator.get_info('EventRuleInfo', event_rule_service.create(params))

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('EventRuleService', metadata) as event_rule_service:
            return self.locator.get_info('EventRuleInfo', event_rule_service.update(params))

    def change_order(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('EventRuleService', metadata) as event_rule_service:
            return self.locator.get_info('EventRuleInfo', event_rule_service.change_order(params))

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('EventRuleService', metadata) as event_rule_service:
            event_rule_service.delete(params)
            return self.locator.get_info('EmptyInfo')

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('EventRuleService', metadata) as event_rule_service:
            return self.locator.get_info('EventRuleInfo', event_rule_service.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('EventRuleService', metadata) as event_rule_service:
            event_rule_vos, total_count = event_rule_service.list(params)
            return self.locator.get_info('EventRulesInfo',
                                         event_rule_vos,
                                         total_count,
                                         minimal=self.get_minimal(params))

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('EventRuleService', metadata) as event_rule_service:
            return self.locator.get_info('StatisticsInfo', event_rule_service.stat(params))



from spaceone.api.monitoring.v1 import event_pb2, event_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class Event(BaseAPI, event_pb2_grpc.EventServicer):

    pb2 = event_pb2
    pb2_grpc = event_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('EventService', metadata) as event_service:
            event_service.create(params)
            return self.locator.get_info('EmptyInfo')

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('EventService', metadata) as event_service:
            return self.locator.get_info('EventInfo', event_service.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('EventService', metadata) as event_service:
            event_vos, total_count = event_service.list(params)
            return self.locator.get_info('EventsInfo',
                                         event_vos,
                                         total_count,
                                         minimal=self.get_minimal(params))

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('EventService', metadata) as event_service:
            return self.locator.get_info('StatisticsInfo', event_service.stat(params))



from spaceone.api.monitoring.v1 import metric_pb2
from spaceone.core.pygrpc.message_type import *

__all__ = ['MetricsInfo', 'MetricDataInfo']


def MetricDataInfo(metric_data):
    info = {
        'labels': change_list_value_type(metric_data.get('labels', [])),
        'values': change_struct_type(metric_data['values']),
        'domain_id': metric_data['domain_id']
    }

    return metric_pb2.MetricDataInfo(**info)


def MetricInfo(metric):
    info = {
        'key': metric['key'],
        'name': metric['name'],
    }

    if 'group' in metric:
        info.update({'group': metric['group']})

    if 'unit' in metric:
        info.update({'unit': change_struct_type(metric['unit'])})

    if 'metric_query' in metric:
        info.update({'metric_query': change_struct_type(metric['metric_query'])})

    return metric_pb2.MetricInfo(**info)


def MetricsInfo(metrics_info):
    info = {
        'metrics': list(map(MetricInfo, metrics_info['metrics'])) if metrics_info['metrics'] else None,
        'available_resources': change_struct_type(metrics_info['available_resources']),
        'domain_id': metrics_info['domain_id']
    }

    return metric_pb2.MetricsInfo(**info)



import functools
from spaceone.api.monitoring.v1 import project_alert_config_pb2
from spaceone.core import utils
from spaceone.monitoring.model.project_alert_config_model import ProjectAlertConfig, AlertOptions
from spaceone.monitoring.info.escalation_policy_info import EscalationPolicyInfo

__all__ = ['ProjectAlertConfigInfo', 'ProjectAlertConfigsInfo']


def AlertOptionsInfo(vo: AlertOptions):
    if vo:
        info = {
            'notification_urgency': vo.notification_urgency,
            'recovery_mode': vo.recovery_mode,
        }

        return project_alert_config_pb2.AlertOptions(**info)
    else:
        return None


def ProjectAlertConfigInfo(project_alert_config_vo: ProjectAlertConfig, minimal=False):
    info = {
        'project_id': project_alert_config_vo.project_id,
        'options': AlertOptionsInfo(project_alert_config_vo.options)
    }

    if not minimal:
        info.update({
            'domain_id': project_alert_config_vo.domain_id,
            'created_at': utils.datetime_to_iso8601(project_alert_config_vo.created_at)
        })

        if project_alert_config_vo.escalation_policy:
            info['escalation_policy_info'] = EscalationPolicyInfo(project_alert_config_vo.escalation_policy,
                                                                  minimal=True)

    return project_alert_config_pb2.ProjectAlertConfigInfo(**info)


def ProjectAlertConfigsInfo(project_alert_config_vos, total_count, **kwargs):
    return project_alert_config_pb2.ProjectAlertConfigsInfo(results=list(
        map(functools.partial(ProjectAlertConfigInfo, **kwargs), project_alert_config_vos)), total_count=total_count)



from spaceone.core.error import *


class ERROR_CLOSED_MAINTENANCE_WINDOW(ERROR_INVALID_ARGUMENT):
    _message = 'Closed maintenance window cannot be updated. (maintenance_window_id = {maintenance_window_id})'



import logging
import functools
from typing import List

from spaceone.core import utils
from spaceone.core.manager import BaseManager
from spaceone.monitoring.manager import IdentityManager
from spaceone.monitoring.model.event_rule_model import EventRule, EventRuleCondition

_LOGGER = logging.getLogger(__name__)


class EventRuleManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.event_rule_model: EventRule = self.locator.get_model('EventRule')
        self.identity_mgr: IdentityManager = self.locator.get_manager('IdentityManager')
        self._service_account_info = {}

    def create_event_rule(self, params):
        def _rollback(event_rule_vo: EventRule):
            _LOGGER.info(f'[create_event_rule._rollback] '
                         f'Delete event rule : {event_rule_vo.name} '
                         f'({event_rule_vo.event_rule_id})')
            event_rule_vo.delete()

        event_rule_vo: EventRule = self.event_rule_model.create(params)
        self.transaction.add_rollback(_rollback, event_rule_vo)

        return event_rule_vo

    def update_event_rule(self, params):
        event_rule_vo: EventRule = self.get_event_rule(params['event_rule_id'], params['domain_id'])
        return self.update_event_rule_by_vo(params, event_rule_vo)

    def update_event_rule_by_vo(self, params, event_rule_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[update_event_rule_by_vo._rollback] Revert Data : '
                         f'{old_data["event_rule_id"]}')
            event_rule_vo.update(old_data)

        self.transaction.add_rollback(_rollback, event_rule_vo.to_dict())

        return event_rule_vo.update(params)

    def delete_event_rule(self, event_rule_id, domain_id):
        event_rule_vo: EventRule = self.get_event_rule(event_rule_id, domain_id)
        self.delete_event_rule_by_vo(event_rule_vo)

    def delete_event_rule_by_vo(self, event_rule_vo):
        event_rule_vo.delete()

    def get_event_rule(self, event_rule_id, domain_id, only=None):
        return self.event_rule_model.get(event_rule_id=event_rule_id, domain_id=domain_id, only=only)

    def list_event_rules(self, query={}):
        return self.event_rule_model.query(**query)

    def stat_event_rules(self, query):
        return self.event_rule_model.stat(**query)

    def change_event_data(self, event_data, project_id, domain_id):
        event_rule_vos: List[EventRule] = self._get_project_event_rules(project_id, domain_id)

        for event_rule_vo in event_rule_vos:
            is_match = self._change_event_data_by_event_rule(event_data, event_rule_vo)

            if is_match:
                event_data = self._change_event_data_with_actions(event_data, event_rule_vo.actions, domain_id)

            if is_match and event_rule_vo.options.stop_processing:
                break

        # TODO: Check Global Event Rule

        return event_data

    def _change_event_data_with_actions(self, event_data, actions, domain_id):
        for action, value in actions.items():
            if action == 'change_project':
                event_data['project_id'] = value

            if action == 'change_assignee':
                event_data['assignee'] = value

            if action == 'change_urgency':
                event_data['urgency'] = value

            if action == 'add_project_dependency':
                event_data['project_dependencies'] = value

            if action == 'add_responder':
                event_data['responders'] = value

            if action == 'add_additional_info':
                event_data['additional_info'] = event_data.get('additional_info', {})
                event_data['additional_info'].update(value)

            if action == 'no_notification':
                event_data['no_notification'] = value

            if action == 'match_service_account':
                source = value['source']
                target_key = value['target']
                target_value = utils.get_dict_value(event_data, source)
                if target_value:
                    service_account_info = self._get_service_account(target_key, target_value, domain_id)
                    if service_account_info:
                        event_data['project_id'] = service_account_info.get('project_info', {}).get('project_id')

        return event_data

    def _change_event_data_by_event_rule(self, event_data, event_rule_vo: EventRule):
        conditions_policy = event_rule_vo.conditions_policy

        if conditions_policy == 'ALWAYS':
            return True
        else:
            results = list(map(functools.partial(self._check_condition, event_data),
                               event_rule_vo.conditions))

            if conditions_policy == 'ALL':
                return all(results)
            else:
                return any(results)

    @staticmethod
    def _check_condition(event_data, condition: EventRuleCondition):
        event_value = utils.get_dict_value(event_data, condition.key)
        condition_value = condition.value
        operator = condition.operator

        if event_value is None:
            return False

        if operator == 'eq':
            if event_value == condition_value:
                return True
            else:
                return False
        elif operator == 'contain':
            if event_value.lower().find(condition_value.lower()) >= 0:
                return True
            else:
                return False
        elif operator == 'not':
            if event_value != condition_value:
                return True
            else:
                return False
        elif operator == 'not_contain':
            if event_value.lower().find(condition_value.lower()) < 0:
                return True
            else:
                return False

        return False

    def _get_project_event_rules(self, project_id, domain_id):
        query = {
            'filter': [
                {
                    'k': 'project_id',
                    'v': project_id,
                    'o': 'eq'
                },
                {
                    'k': 'domain_id',
                    'v': domain_id,
                    'o': 'eq'
                }
            ],
            'sort': {
                'key': 'order'
            }
        }

        event_rule_vos, total_count = self.list_event_rules(query)
        return event_rule_vos

    def _get_service_account(self, target_key, target_value, domain_id):
        if f'{domain_id}:{target_key}:{target_value}' in self._service_account_info:
            return self._service_account_info[f'{domain_id}:{target_key}:{target_value}']

        query = {
            'filter': [
                {'k': target_key, 'v': target_value, 'o': 'eq'}
            ],
            'only': ['service_account_id', 'project_info']
        }

        response = self.identity_mgr.list_service_accounts(query, domain_id)
        results = response.get('results', [])
        total_count = response.get('total_count', 0)

        service_account_info = None
        if total_count > 0:
            service_account_info = results[0]

        self._service_account_info[f'{domain_id}:{target_key}:{target_value}'] = service_account_info
        return service_account_info



import logging
import concurrent.futures

from spaceone.core.service import *
from spaceone.core.utils import get_dict_value, random_string

from spaceone.monitoring.error import *
from spaceone.monitoring.manager.inventory_manager import InventoryManager
from spaceone.monitoring.manager.secret_manager import SecretManager
from spaceone.monitoring.manager.data_source_manager import DataSourceManager
from spaceone.monitoring.manager.data_source_plugin_manager import DataSourcePluginManager
from spaceone.monitoring.conf.global_conf import *

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class MetricService(BaseService):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.inventory_mgr: InventoryManager = self.locator.get_manager('InventoryManager')
        self.secret_mgr: SecretManager = self.locator.get_manager('SecretManager')
        self.data_source_mgr: DataSourceManager = self.locator.get_manager('DataSourceManager')
        self.ds_plugin_mgr: DataSourcePluginManager = self.locator.get_manager('DataSourcePluginManager')

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['data_source_id', 'resources', 'domain_id'])
    def list(self, params):
        """ Get resource's metrics

        Args:
            params (dict): {
                'data_source_id': 'str',
                'resources': 'list',
                'domain_id': 'str'
            }

        Returns:
            metrics (list)
        """
        data_source_id = params['data_source_id']
        resources = params['resources']
        domain_id = params['domain_id']

        data_source_vo = self.data_source_mgr.get_data_source(data_source_id, domain_id)
        self._check_data_source_state(data_source_vo)

        plugin_info = data_source_vo.plugin_info
        plugin_metadata = plugin_info.metadata
        self._check_plugin_metadata(plugin_metadata, params, data_source_id)
        self.plugin_initialize(data_source_vo)

        metrics_dict = {}
        and_metric_keys = []

        response = {
            'metrics': [],
            'available_resources': {},
            'domain_id': domain_id
        }

        for resource_id in resources:
            response['available_resources'][resource_id] = False

        required_keys = plugin_metadata.get('required_keys')
        cloud_services_info = self.inventory_mgr.list_resources(resources, required_keys, domain_id)

        list_metric_params = []
        for cloud_service_info in cloud_services_info:
            secret = self.secret_mgr.get_secret_from_resource(cloud_service_info, data_source_vo, domain_id)
            secret_data = self.secret_mgr.get_secret_data(secret.get('secret_id', ''), domain_id)

            query = self.get_query_from_cloud_service(cloud_service_info, plugin_info)

            list_metric_params.append({
                'schema': secret.get('schema'),
                'options': plugin_info.options,
                'secret_data': secret_data,
                'query': query,
                'cloud_service_id': cloud_service_info['cloud_service_id'],
                'data_source_vo': data_source_vo,
                'domain_id': domain_id,
                'metrics_dict': metrics_dict,
                'and_metric_keys': and_metric_keys
            })

        metric_responses = []
        with concurrent.futures.ThreadPoolExecutor(max_workers=MAX_CONCURRENT_WORKER) as executor:
            future_executors = []

            for _params in list_metric_params:
                future_executors.append(executor.submit(self.list_metrics_info, _params))

            for future in concurrent.futures.as_completed(future_executors):
                metric_responses.append(future.result())

        response = self._merge_metric(metric_responses, domain_id)
        return response

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['data_source_id', 'metric_query', 'metric', 'start', 'end', 'domain_id'])
    def get_data(self, params):
        """ Get resource's metric data

        Args:
            params (dict): {
                'data_source_id': 'str',
                'metric_query': 'dict',
                'metric': 'str',
                'start': 'str',
                'end': 'str',
                'period': 'int',
                'stat': 'str',
                'domain_id': 'str'
            }

        Returns:
            metric_data (list)
        """
        data_source_id = params['data_source_id']
        metric_query = params['metric_query']
        domain_id = params['domain_id']

        data_source_vo = self.data_source_mgr.get_data_source(data_source_id, domain_id)
        self._check_data_source_state(data_source_vo)

        plugin_metadata = data_source_vo.plugin_info.metadata
        self._check_plugin_metadata(plugin_metadata, params, data_source_id)
        self.plugin_initialize(data_source_vo)

        response = {
            'labels': [],
            'values': {},
            'domain_id': domain_id
        }

        required_keys = plugin_metadata.get('required_keys')
        resource_ids = self.get_resource_ids_from_metric_query(metric_query)
        resources_info = self.inventory_mgr.list_resources(resource_ids, required_keys, domain_id)
        resources_chunks = self.list_chunk_resources(resources_info, metric_query, data_source_vo, domain_id)

        _LOGGER.debug(f"[get_data] chunk_resources: {resources_chunks}")

        metric_data_params = self.set_metric_data_params(params)

        for chunk_resources in resources_chunks.values():
            metric_data_params.update({
                'secret_data': chunk_resources.get('secret_data'),
                'metric_query': chunk_resources.get('metric_query'),
                'options': chunk_resources.get('options', {})
            })

            if 'schema' in chunk_resources:
                metric_data_params.update({'schema': chunk_resources.get('schema')})

            metric_data_response = self.get_metric_data(metric_data_params)

            if not response['labels'] and metric_data_response.get('labels', []):
                response['labels'] = metric_data_response['labels']

            if values := metric_data_response.get('values'):
                response['values'].update(values)

        return response

    def list_chunk_resources(self, resources, metric_query, data_source_vo, domain_id):
        """
        chunk_resources(dict): {
            'provider.region_code.secret_id.random_key': {
                'schema': 'str',
                'secret_data': 'dict',
                'region_name': 'str',
                'options': 'dict',
                'metric_query': 'dict'     # MAX_REQUEST_LIMIT
            },
            ...
        }
        """
        chunk_resources = {}
        provider = data_source_vo.plugin_info['provider']

        for resource in resources:
            cloud_service_id = resource['cloud_service_id']
            if cloud_service_id in metric_query:
                region_code = self.get_region_from_resource(resource)
                secret = self.secret_mgr.get_secret_from_resource(resource, data_source_vo, domain_id)
                chunk_key = self._generate_chunk_key(provider, region_code, secret.get('secret_id', ''))

                metric_query[cloud_service_id]['region_name'] = region_code
                chunk_resource = {cloud_service_id: metric_query[cloud_service_id]}

                if chunk_key in chunk_resources:
                    chunk_info = chunk_resources[chunk_key]

                    if 'region_name' not in chunk_info:
                        chunk_info.update({'region_name': region_code})

                    if 'secret_data' not in chunk_info:
                        chunk_info.update({'secret_data': self.secret_mgr.get_secret_data(secret['secret_id'], domain_id)})

                    if 'schema' not in chunk_info:
                        chunk_info.update({'schema': secret.get('schema')})

                    _metric_query = chunk_info['metric_query']

                    if len(_metric_query) >= MAX_REQUEST_LIMIT:
                        new_chunk_key = f'{chunk_key}.{random_string(1)}'
                        # move chunk to new key
                        chunk_resources[new_chunk_key] = chunk_info
                        # generate a new chunk
                        chunk_resources[chunk_key] = {
                            'region_name': region_code,
                            'secret_data': self.secret_mgr.get_secret_data(secret['secret_id'], domain_id),
                            'metric_query': chunk_resource,
                        }
                    else:
                        _metric_query[cloud_service_id] = metric_query[cloud_service_id]
                else:
                    # generate a new chunk
                    chunk_resources[chunk_key] = {
                        'region_name': region_code,
                        'secret_data': self.secret_mgr.get_secret_data(secret['secret_id'], domain_id),
                        'metric_query': chunk_resource,
                    }

        return chunk_resources

    def plugin_initialize(self, data_source_vo):
        endpoint = self.ds_plugin_mgr.get_data_source_plugin_endpoint_by_vo(data_source_vo)
        self.ds_plugin_mgr.initialize(endpoint)

    def get_metric_data(self, params):
        metric_data_info = {'labels': [], 'values': {}}

        try:
            metric_data_info = self.ds_plugin_mgr.get_metric_data(params)
        except Exception as e:
            print(e)

        return metric_data_info

    def list_metrics_info(self, params):
        schema = params.get('schema')
        secret_data = params['secret_data']
        cloud_service_id = params['cloud_service_id']
        query = params.get('query', {})
        options = params.get('options', {})

        try:
            metrics_info = self.ds_plugin_mgr.list_metrics(schema, options, secret_data, query)
            _LOGGER.debug(f'[list_metric_info] metrics_info: {metrics_info}')
            return {
                'metrics': metrics_info.get('metrics', []),
                'resource_id': cloud_service_id
            }
            # return response
        except Exception as e:
            _LOGGER.error(f'[list_metrics]: {e}')
            return {}

    def _merge_metric(self, metric_responses, domain_id):
        response = {
            'metrics': [],
            'available_resources': {},
            'domain_id': domain_id
        }

        available_resources, metric_responses, cloud_service_metric_set_info = self.set_preload_metric(metric_responses)

        _LOGGER.debug(f'available_resources: {available_resources}')
        _LOGGER.debug(f'metric_responses: {metric_responses}')
        _LOGGER.debug(f'cloud_service_metric_set_info: {cloud_service_metric_set_info}')

        if len(metric_responses) == 1:
            response['metrics'] = metric_responses[0]['metrics']
        elif len(metric_responses) > 1:
            _intersected_metric_keys = self._intersection_metric_keys(cloud_service_metric_set_info)

            response_metrics = []
            for metric_response in metric_responses:
                for _metric in metric_response['metrics']:
                    if _metric['key'] in _intersected_metric_keys:
                        match_key = False
                        for response_metric in response_metrics:
                            if response_metric['key'] == _metric['key']:
                                response_metric['metric_query'].update(_metric['metric_query'])
                                match_key = True
                                break

                        if match_key is False:
                            response_metrics.append(_metric)

            response['metrics'] = response_metrics

        response['available_resources'] = available_resources

        _LOGGER.debug(response)
        return response

    @staticmethod
    def set_preload_metric(metric_responses):
        available_resources = {}
        cloud_service_metric_set_info = {}

        for metric_response in metric_responses:
            if cloud_service_id := metric_response.get('resource_id'):
                available_resources[cloud_service_id] = True

                _metric_keys = []
                for _metric in metric_response['metrics']:
                    _metric_keys.append(_metric['key'])

                    _metric_query = _metric['metric_query']
                    _metric['metric_query'] = {cloud_service_id: _metric_query}

                cloud_service_metric_set_info[cloud_service_id] = _metric_keys

        return available_resources, metric_responses, cloud_service_metric_set_info

    @staticmethod
    def _intersection_metric_keys(cloud_service_metric_set_info):
        target_metrics = []

        for metrics in cloud_service_metric_set_info.values():
            if metrics and target_metrics:
                target_metrics = list(set(metrics) & set(target_metrics))

            if metrics and not target_metrics:
                target_metrics = metrics

        return target_metrics

    @staticmethod
    def get_resource_ids_from_metric_query(metric_query):
        return [resource_id for resource_id in metric_query.keys()]

    @staticmethod
    def set_metric_data_params(params):
        return_params = {
            'metric': params['metric'],
            'start': params['start'],
            'end': params['end'],
        }

        if 'period' in params:
            return_params['period'] = params['period']

        if 'stat' in params:
            return_params['stat'] = params['stat']

        return return_params

    @staticmethod
    def _generate_chunk_key(provider, region_code, secret_id):
        return f"{provider}.{region_code}.{secret_id}"

    @staticmethod
    def get_region_from_resource(resource):
        return resource.get('region_code', '')

    @staticmethod
    def get_account_from_resource(resource):
        return resource.get('account', '')

    @staticmethod
    def _check_data_source_state(data_source_vo):
        if data_source_vo.state == 'DISABLED':
            raise ERROR_DATA_SOURCE_STATE_DISABLED(data_source_id=data_source_vo.data_source_id)

    @staticmethod
    def _check_resource_type(plugin_metadata, resource_type):
        supported_resource_type = plugin_metadata['supported_resource_type']

        if resource_type not in supported_resource_type:
            raise ERROR_NOT_SUPPORT_RESOURCE_TYPE(supported_resource_type=supported_resource_type)

    @staticmethod
    def _check_plugin_metadata(metadata, params, data_source_id):
        if 'supported_stat' in metadata and 'stat' in params:
            if params['stat'] not in metadata['supported_stat']:
                raise ERROR_NOT_SUPPORT_METRIC_STAT(stat=params['stat'])

        if 'required_keys' in metadata:
            if not metadata['required_keys']:
                raise ERROR_NOT_FOUND_REQUIRED_KEY(data_source_id=data_source_id)
        else:
            raise ERROR_NOT_FOUND_REQUIRED_KEY(data_source_id=data_source_id)

    @staticmethod
    def _merge_metric_keys(metrics_info, metrics_dict, and_metric_keys):
        for metric_info in metrics_info.get('metrics', []):
            if 'key' in metric_info:
                metric_key = metric_info['key']

                if metric_key not in and_metric_keys:
                    metrics_dict[metric_key] = metric_info
                    and_metric_keys.append(metric_key)

        return {'metrics_dict': metrics_dict, 'and_metric_keys': and_metric_keys}

    @staticmethod
    def get_required_key(metadata):
        required_keys = metadata.get('required_keys', [])

        if not required_keys:
            raise

        return required_keys[0]

    @staticmethod
    def get_query_from_cloud_service(cloud_service_info, plugin_info):
        metadata = plugin_info.metadata
        required_keys = metadata.get('required_keys', [])

        if required_keys:
            query_key = required_keys[0]
            return get_dict_value(cloud_service_info, query_key, default_value={})
        else:
            raise ERROR_REQUIRED_KEYS_NOT_EXISTS(plugin_id=plugin_info.plugin_id)
